@@ -29,14 +29,17 @@
 
 #include <config.h>
 
+#include "urlinfo.h"
+
 #include <KLocalizedString>
 #include <Kdelibs4ConfigMigrator>
-
 #include <kaboutdata.h>
 #include <kmessagebox.h>
 #include <ktexteditor/cursor.h>
+#include <kcrash.h>
 
 #include <QApplication>
+#include <QElapsedTimer>
 #include <QCommandLineParser>
 #include <QCommandLineOption>
 #include <QFileInfo>
@@ -66,26 +69,92 @@ Q_LOGGING_CATEGORY(APP, "kdevelop.app")
 #include <util/path.h>
 
 #include "kdevideextension.h"
+#if KDEVELOP_SINGLE_APP
+#include "qtsingleapplication.h"
+#endif
 
 #include <iostream>
 
-#include "splash.h"
-
 #ifdef Q_OS_MAC
 #include <CoreFoundation/CoreFoundation.h>
 #endif
 
 using namespace KDevelop;
 
-class KDevelopApplication: public QApplication
+namespace {
+
+#if KDEVELOP_SINGLE_APP
+QString serializeOpenFilesMessage(const QVector<UrlInfo> &infos)
+{
+    QByteArray message;
+    QDataStream stream(&message, QIODevice::WriteOnly);
+    stream << QByteArrayLiteral("open");
+    stream << infos;
+    return QString::fromLatin1(message.toHex());
+}
+#endif
+
+void openFiles(const QVector<UrlInfo>& infos)
+{
+    foreach (const UrlInfo& file, infos) {
+        if (!ICore::self()->documentController()->openDocument(file.url, file.cursor)) {
+            qWarning() << i18n("Could not open %1", file.url.toDisplayString(QUrl::PreferLocalFile));
+        }
+    }
+}
+
+}
+
+class KDevelopApplication:
+#if KDEVELOP_SINGLE_APP
+    public SharedTools::QtSingleApplication
+#else
+    public QApplication
+#endif
 {
 public:
     explicit KDevelopApplication(int &argc, char **argv, bool GUIenabled = true)
+#if KDEVELOP_SINGLE_APP
+        : SharedTools::QtSingleApplication(QStringLiteral("KDevelop"), argc, argv)
+#else
         : QApplication(argc, argv, GUIenabled)
+#endif
         {
+#if KDEVELOP_SINGLE_APP
+            Q_UNUSED(GUIenabled);
+#endif
+
             connect(this, &QGuiApplication::saveStateRequest, this, &KDevelopApplication::saveState);
         }
 
+#if KDEVELOP_SINGLE_APP
+public Q_SLOTS:
+    void remoteArguments(const QString &message, QObject *socket)
+    {
+        Q_UNUSED(socket);
+
+        QByteArray ba = QByteArray::fromHex(message.toLatin1());
+        QDataStream stream(ba);
+        QByteArray command;
+        stream >> command;
+
+        qCDebug(APP) << "Received remote command: " << command;
+
+        if (command == "open") {
+            QVector<UrlInfo> infos;
+            stream >> infos;
+            openFiles(infos);
+        } else {
+            qCWarning(APP) << "Unknown remote command: " << command;
+        }
+    }
+
+    void fileOpenRequested(const QString &file)
+    {
+        openFiles({UrlInfo(file)});
+    }
+#endif
+
 private Q_SLOTS:
     void saveState( QSessionManager& sm ) {
         if (KDevelop::Core::self() && KDevelop::Core::self()->sessionController()) {
@@ -100,14 +169,14 @@ private Q_SLOTS:
 /// Tries to find a session identified by @p data in @p sessions.
 /// The @p data may be either a session's name or a string-representation of its UUID.
 /// @return pointer to the session or NULL if nothing appropriate has been found
-static const KDevelop::SessionInfo* findSessionInList( QList<KDevelop::SessionInfo>& sessions, const QString& data )
+static const KDevelop::SessionInfo* findSessionInList( const SessionInfos& sessions, const QString& data )
 {
     // We won't search a session without input data, since that could lead to false-positives
     // with unnamed sessions
     if( data.isEmpty() )
         return 0;
 
-    for( QList<KDevelop::SessionInfo>::const_iterator it = sessions.constBegin(); it != sessions.constEnd(); ++it ) {
+    for( auto it = sessions.constBegin(); it != sessions.constEnd(); ++it ) {
         if ( ( it->name == data ) || ( it->uuid.toString() == data ) ) {
             const KDevelop::SessionInfo& sessionRef = *it;
             return &sessionRef;
@@ -116,42 +185,9 @@ static const KDevelop::SessionInfo* findSessionInList( QList<KDevelop::SessionIn
     return 0;
 }
 
-
-// Represents a file to be opened, consisting of its URL and the cursor to jump to.
-struct UrlInfo
-{
-    // Parses a file path argument and determines its line number and column and full path
-    UrlInfo(QString path = {})
-        : cursor(KTextEditor::Cursor::invalid())
-    {
-        if (!path.startsWith(QLatin1Char('/')) && QFileInfo(path).isRelative()) {
-            path = QDir::currentPath() + QLatin1Char('/') + path;
-        }
-        url = QUrl::fromUserInput(path);
-
-        if (url.isLocalFile() && !QFile::exists(path)) {
-            // Allow opening specific lines in documents, like mydoc.cpp:10
-            // also supports columns, i.e. mydoc.cpp:10:42
-            static const QRegularExpression pattern(QStringLiteral(":(\\d+)(?::(\\d+))?$"));
-            const auto match = pattern.match(path);
-            if (match.isValid()) {
-                path.chop(match.capturedLength());
-                int line = match.captured(1).toInt() - 1;
-                // don't use an invalid column when the line is valid
-                int column = qMax(0, match.captured(2).toInt() - 1);
-                url = QUrl::fromLocalFile(path);
-                cursor = {line, column};
-            }
-        }
-    }
-
-    QUrl url;
-    KTextEditor::Cursor cursor;
-};
-
 /// Performs a DBus call to open the given @p files in the running kdev instance identified by @p pid
 /// Returns the exit status
-static int openFilesInRunningInstance(const QVector<UrlInfo>& files, int pid)
+static int openFilesInRunningInstance(const QVector<UrlInfo>& files, qint64 pid)
 {
     const QString service = QString("org.kdevelop.kdevelop-%1").arg(pid);
     QDBusInterface iface(service, "/org/kdevelop/DocumentController", "org.kdevelop.DocumentController");
@@ -175,10 +211,10 @@ static int openFilesInRunningInstance(const QVector<UrlInfo>& files, int pid)
 
 /// Gets the PID of a running KDevelop instance, eventually asking the user if there is more than one.
 /// Returns -1 in case there are no running sessions.
-static int getRunningSessionPid()
+static qint64 getRunningSessionPid()
 {
-    QList<KDevelop::SessionInfo> candidates;
-    foreach( const KDevelop::SessionInfo& si, KDevelop::SessionController::availableSessionInfo() ) {
+    SessionInfos candidates;
+    foreach( const KDevelop::SessionInfo& si, KDevelop::SessionController::availableSessionInfos() ) {
         if( KDevelop::SessionController::isSessionRunning(si.uuid.toString()) ) {
             candidates << si;
         }
@@ -198,12 +234,12 @@ static int getRunningSessionPid()
     return KDevelop::SessionController::sessionRunInfo(sessionUuid).holderPid;
 }
 
-static QString findSessionId(const QString& session)
+static QString findSessionId(const SessionInfos& availableSessionInfos, const QString& session)
 {
     //If there is a session and a project with the same name, always open the session
     //regardless of the order encountered
     QString projectAsSession;
-    foreach(const KDevelop::SessionInfo& si, KDevelop::SessionController::availableSessionInfo())
+    foreach(const KDevelop::SessionInfo& si, availableSessionInfos)
     {
         if ( session == si.name || session == si.uuid.toString() ) {
             return si.uuid.toString();
@@ -220,7 +256,7 @@ static QString findSessionId(const QString& session)
 
     if (projectAsSession.isEmpty())  {
         QTextStream qerr(stderr);
-        qerr << endl << i18n("Cannot open unknown session %1. See --sessions switch for available sessions or use -cs to create a new one.",
+        qerr << endl << i18n("Cannot open unknown session %1. See `--list-sessions` switch for available sessions or use `-n` to create a new one.",
                              session) << endl;
     }
     return projectAsSession;
@@ -234,6 +270,9 @@ static qint64 findSessionPid(const QString &sessionId)
 
 int main( int argc, char *argv[] )
 {
+    QElapsedTimer timer;
+    timer.start();
+
     // TODO: Maybe generalize, add KDEVELOP_STANDALONE build option
 #if defined(Q_OS_WIN) || defined(Q_OS_MAC)
     qputenv("KDE_FORK_SLAVES", "1"); // KIO slaves will be forked off instead of being started via DBus
@@ -241,7 +280,6 @@ int main( int argc, char *argv[] )
 
     // Useful for valgrind runs, just `export KDEV_DISABLE_JIT=1`
     if (qEnvironmentVariableIsSet("KDEV_DISABLE_JIT")) {
-        qputenv("KDEV_DISABLE_SPLASH", "1");
         qputenv("KDEV_DISABLE_WELCOMEPAGE", "1");
         qputenv("QT_ENABLE_REGEXP_JIT", "0");
     }
@@ -360,6 +398,8 @@ int main( int argc, char *argv[] )
 
     KDevelopApplication app(argc, argv);
 
+    KCrash::initialize();
+
     Kdelibs4ConfigMigrator migrator(QStringLiteral("kdevelop"));
     migrator.setConfigFiles({QStringLiteral("kdeveloprc")});
     migrator.setUiFiles({QStringLiteral("kdevelopui.rc")});
@@ -379,6 +419,8 @@ int main( int argc, char *argv[] )
     parser.addOption(QCommandLineOption{QStringList{"n", "new-session"}, i18n("Open KDevelop with a new session using the given name."), "name"});
     parser.addOption(QCommandLineOption{QStringList{"s", "open-session"}, i18n("Open KDevelop with the given session.\n"
                      "You can pass either hash or the name of the session." ), "session"});
+    parser.addOption(QCommandLineOption{QStringList{"rm", "remove-session"}, i18n("Delete the given session.\n"
+                     "You can pass either hash or the name of the session." ), "session"});
     parser.addOption(QCommandLineOption{QStringList{"ps", "pick-session"}, i18n("Shows all available sessions and lets you select one to open.")});
     parser.addOption(QCommandLineOption{QStringList{"pss", "pick-session-shell"}, i18n("List all available sessions on shell and lets you select one to open.")});
     parser.addOption(QCommandLineOption{QStringList{"l", "list-sessions"}, i18n("List available sessions and quit.")});
@@ -388,7 +430,13 @@ int main( int argc, char *argv[] )
                      "The binary that should be debugged must follow - including arguments.\n"
                      "Example: kdevelop --debug gdb myapp --foo bar"), "debugger"});
 
-    // TODO: What does that do?
+    // this is used by the 'kdevelop!' script to retrieve the pid of a KDEVELOP
+    // instance. When this is called, then we should just print the PID on the
+    // standard-output. If a session is specified through open-session, then
+    // we should return the PID of that session. Otherwise, if only a single
+    // session is running, then we should just return the PID of that session.
+    // Otherwise, we should print a command-line session-chooser dialog ("--pss"),
+    // which only shows the running sessions, and the user can pick one.
     parser.addOption(QCommandLineOption{QStringList{"pid"}});
 
     parser.addPositionalArgument("files", i18n( "Files to load" ), "[FILE...]");
@@ -404,7 +452,7 @@ int main( int argc, char *argv[] )
         QTextStream qout(stdout);
         qout << endl << ki18n("Available sessions (use '-s HASH' or '-s NAME' to open a specific one):").toString() << endl << endl;
         qout << QString("%1").arg(ki18n("Hash").toString(), -38) << '\t' << ki18n("Name: Opened Projects").toString() << endl;
-        foreach(const KDevelop::SessionInfo& si, KDevelop::SessionController::availableSessionInfo())
+        foreach(const KDevelop::SessionInfo& si, KDevelop::SessionController::availableSessionInfos())
         {
             if ( si.name.isEmpty() && si.projects.isEmpty() ) {
                 continue;
@@ -424,11 +472,21 @@ int main( int argc, char *argv[] )
     foreach (const QString &file, parser.positionalArguments()) {
         initialFiles.append(UrlInfo(file));
     }
-    if ( ! initialFiles.isEmpty() && ! parser.isSet("new-session") )
-    {
-        int pid = -1;
+
+    const auto availableSessionInfos = KDevelop::SessionController::availableSessionInfos();
+
+    if (!initialFiles.isEmpty() && !parser.isSet("new-session")) {
+#if KDEVELOP_SINGLE_APP
+        if (app.isRunning()) {
+            bool success = app.sendMessage(serializeOpenFilesMessage(initialFiles));
+            if (success) {
+                return 0;
+            }
+        }
+#else
+        qint64 pid = -1;
         if (parser.isSet("open-session")) {
-            const QString session = findSessionId(parser.value("open-session"));
+            const QString session = findSessionId(availableSessionInfos, parser.value("open-session"));
             if (session.isEmpty()) {
                 return 1;
             } else if (KDevelop::SessionController::isSessionRunning(session)) {
@@ -441,16 +499,24 @@ int main( int argc, char *argv[] )
             return openFilesInRunningInstance(initialFiles, pid);
         }
         // else there are no running sessions, and the generated list of files will be opened below.
+#endif
     }
 
     // if empty, restart kdevelop with last active session, see SessionController::defaultSessionId
     QString session;
 
-    if(parser.isSet("pss"))
+    uint nRunningSessions = 0;
+    foreach(const KDevelop::SessionInfo& si, availableSessionInfos)
+        if(KDevelop::SessionController::isSessionRunning(si.uuid.toString()))
+            ++nRunningSessions;
+
+    // also show the picker dialog when a pid shall be retrieved and multiple
+    // sessions are running.
+    if(parser.isSet("pss") || (parser.isSet("pid") && !parser.isSet("open-session") && !parser.isSet("ps") && nRunningSessions > 1))
     {
         QTextStream qerr(stderr);
-        QList<KDevelop::SessionInfo> candidates;
-        foreach(const KDevelop::SessionInfo& si, KDevelop::SessionController::availableSessionInfo())
+        SessionInfos candidates;
+        foreach(const KDevelop::SessionInfo& si, availableSessionInfos)
             if( (!si.name.isEmpty() || !si.projects.isEmpty() || parser.isSet("pid")) &&
                 (!parser.isSet("pid") || KDevelop::SessionController::isSessionRunning(si.uuid.toString())))
                 candidates << si;
@@ -499,7 +565,7 @@ int main( int argc, char *argv[] )
     } else if ( parser.isSet("new-session") )
     {
         session = parser.value("new-session");
-        foreach(const KDevelop::SessionInfo& si, KDevelop::SessionController::availableSessionInfo())
+        foreach(const KDevelop::SessionInfo& si, availableSessionInfos)
         {
             if ( session == si.name ) {
                 QTextStream qerr(stderr);
@@ -509,16 +575,41 @@ int main( int argc, char *argv[] )
         }
         // session doesn't exist, we can create it
     } else if ( parser.isSet("open-session") ) {
-        session = findSessionId(parser.value("open-session"));
+        session = findSessionId(availableSessionInfos, parser.value("open-session"));
         if (session.isEmpty()) {
             return 1;
         }
+    } else if ( parser.isSet("remove-session") )
+    {
+        session = parser.value("remove-session");
+        auto si = findSessionInList(availableSessionInfos, session);
+        if (!si) {
+            QTextStream qerr(stderr);
+            qerr << endl << i18n("No session with the name %1 exists.", session) << endl;
+            return 1;
+        }
+
+        auto sessionLock = KDevelop::SessionController::tryLockSession(si->uuid.toString());
+        if (!sessionLock.lock) {
+            QTextStream qerr(stderr);
+            qerr << endl << i18n("Could not lock session %1 for deletion.", session) << endl;
+            return 1;
+        }
+        KDevelop::SessionController::deleteSessionFromDisk(sessionLock.lock);
+        QTextStream qout(stdout);
+        qout << endl << i18n("Session with name %1 was successfully removed.", session) << endl;
+        return 0;
     }
 
-    QList<KDevelop::SessionInfo> sessions = KDevelop::SessionController::availableSessionInfo();
-    const KDevelop::SessionInfo* sessionData = findSessionInList( sessions, session );
-
     if(parser.isSet("pid")) {
+        if (session.isEmpty())
+        {   // just pick the first running session
+            foreach(const KDevelop::SessionInfo& si, availableSessionInfos)
+                if(KDevelop::SessionController::isSessionRunning(si.uuid.toString()))
+                    session = si.uuid.toString();
+        }
+        const KDevelop::SessionInfo* sessionData = findSessionInList(availableSessionInfos, session);
+
         if( !sessionData ) {
             qCritical() << "session not given or does not exist";
             return 5;
@@ -537,13 +628,7 @@ int main( int argc, char *argv[] )
 
     KDevIDEExtension::init();
 
-    KDevSplashScreen* splash = nullptr;
-    if (!QProcessEnvironment::systemEnvironment().contains("KDEV_DISABLE_SPLASH")) {
-        splash = new KDevSplashScreen;
-        splash->show();
-    }
-
-    if(!Core::initialize(splash, Core::Default, session))
+    if(!Core::initialize(nullptr, Core::Default, session))
         return 5;
 
     // register a DBUS service for this process, so that we can open files in it from other invocations
@@ -634,13 +719,19 @@ int main( int argc, char *argv[] )
 
         core->runControllerInternal()->execute("debug", launch);
     } else {
-        foreach ( const UrlInfo& file, initialFiles ) {
-            if(!core->documentController()->openDocument(file.url, file.cursor)) {
-                qWarning() << i18n("Could not open %1", file.url.toDisplayString(QUrl::PreferLocalFile));
-            }
-        }
+        openFiles(initialFiles);
     }
 
+#if KDEVELOP_SINGLE_APP
+    // Set up remote arguments.
+    QObject::connect(&app, &SharedTools::QtSingleApplication::messageReceived,
+                     &app, &KDevelopApplication::remoteArguments);
+
+    QObject::connect(&app, &SharedTools::QtSingleApplication::fileOpenRequest,
+                     &app, &KDevelopApplication::fileOpenRequested);
+#endif
+
+
 #ifdef WITH_WELCOMEPAGE
     // make it possible to disable the welcome page, useful for valgrind runs e.g.
     if (!qEnvironmentVariableIsSet("KDEV_DISABLE_WELCOMEPAGE")) {
@@ -648,6 +739,8 @@ int main( int argc, char *argv[] )
     }
 #endif
 
-    qCDebug(APP) << "Done startup";
+    qCDebug(APP) << "Done startup" << "- took:" << timer.elapsed() << "ms";
+    timer.invalidate();
+
     return app.exec();
 }
