@@ -57,6 +57,7 @@ using KDevelop::AutoTestShell;
 namespace GDBDebugger {
 
 QUrl findExecutable(const QString& name)
+
 {
     QFileInfo info(qApp->applicationDirPath()  + "/unittests/" + name);
     Q_ASSERT(info.exists());
@@ -254,14 +255,18 @@ private:
     } while(0)
 
 #define COMPARE_DATA(index, expected) \
-    compareData((index), (expected), __FILE__, __LINE__)
+    do { if(!compareData((index), (expected), __FILE__, __LINE__)) return; } while (0)
 
-void compareData(QModelIndex index, QString expected, const char *file, int line)
+bool compareData(QModelIndex index, QString expected, const char *file, int line)
 {
     QString s = index.model()->data(index, Qt::DisplayRole).toString();
     if (s != expected) {
-        QFAIL(qPrintable(QString("'%0' didn't match expected '%1' in %2:%3").arg(s).arg(expected).arg(file).arg(line)));
+        QTest::qFail(qPrintable(QString("'%0' didn't match expected '%1' in %2:%3")
+                                .arg(s).arg(expected).arg(file).arg(line)),
+                     file, line);
+        return false;
     }
+    return true;
 }
 
 static const QString debugeeFileName = findSourceFile("debugee.cpp");
@@ -437,23 +442,26 @@ void GdbTest::testUpdateBreakpoint()
     TestDebugSession *session = new TestDebugSession;
     TestLaunchConfiguration cfg;
 
+    // breakpoint 1: line 29
     KDevelop::Breakpoint * b = breakpoints()->addCodeBreakpoint(QUrl::fromLocalFile(debugeeFileName), 28);
     QCOMPARE(breakpoints()->rowCount(), 1);
 
     session->startProgram(&cfg, m_iface);
 
+    // breakpoint 2: line 28
     //insert custom command as user might do it using GDB console
     session->addCommand(new UserCommand(GDBMI::NonMI, "break "+debugeeFileName+":28"));
 
-    WAIT_FOR_STATE(session, DebugSession::PausedState);
-    QTest::qWait(100);
+    WAIT_FOR_STATE_AND_IDLE(session, DebugSession::PausedState); // stop at line 28
     session->stepInto();
-    WAIT_FOR_STATE(session, DebugSession::PausedState);
+    WAIT_FOR_STATE_AND_IDLE(session, DebugSession::PausedState); // stop after step
     QCOMPARE(breakpoints()->rowCount(), 2);
     b = breakpoints()->breakpoint(1);
     QCOMPARE(b->url(), QUrl::fromLocalFile(debugeeFileName));
     QCOMPARE(b->line(), 27);
     session->run();
+    WAIT_FOR_STATE(session, DebugSession::PausedState); // stop at line 29
+    session->run();
     WAIT_FOR_STATE(session, DebugSession::EndedState);
 }
 
@@ -1994,29 +2002,31 @@ void GdbTest::testPathWithSpace()
 bool GdbTest::waitForState(GDBDebugger::DebugSession *session, DebugSession::DebuggerState state,
                             const char *file, int line, bool waitForIdle)
 {
-    QPointer<GDBDebugger::DebugSession> s(session); //session can get deleted in DebugController
+    QPointer<DebugSession> s(session); //session can get deleted in DebugController
     QTime stopWatch;
     stopWatch.start();
-    while (s.data()->state() != state || (waitForIdle && s->stateIsOn(s_dbgBusy))) {
+
+    // legacy behavior for tests that implicitly may require waiting for idle,
+    // but which were written before waitForIdle was added
+    waitForIdle = waitForIdle || state != DebugSession::EndedState;
+
+    while (s && (s->state() != state || (waitForIdle && s->stateIsOn(s_dbgBusy)))) {
         if (stopWatch.elapsed() > 5000) {
-            qWarning() << "current state" << s.data()->state() << "waiting for" << state;
+            qWarning() << "current state" << s->state() << "waiting for" << state;
             QTest::qFail(qPrintable(QString("Timeout before reaching state %0").arg(state)),
-                file, line);
+                         file, line);
             return false;
         }
         QTest::qWait(20);
-        if (!s) {
-            if (state == DebugSession::EndedState)
-                break;
-            QTest::qFail(qPrintable(QString("Session ended before reaching state %0").arg(state)),
-                file, line);
-            return false;
-        }
     }
-    if (!waitForIdle && state != DebugSession::EndedState) {
-        // legacy behavior for tests that implicitly may require waiting for idle,
-        // but which were written before waitForIdle was added
-        QTest::qWait(100);
+
+    // NOTE: don't wait anymore after leaving the loop. Waiting re-enters event loop and
+    // may change session state.
+
+    if (!s && state != DebugSession::EndedState) {
+        QTest::qFail(qPrintable(QString("Session ended before reaching state %0").arg(state)),
+                     file, line);
+        return false;
     }
 
     qDebug() << "Reached state " << state << " in " << file << ':' << line;
