@@ -55,22 +55,6 @@ struct ImplementsInfo
     QString templatePrefix;
 };
 
-//TODO replace this with clang_Type_getTemplateArgumentAsType when that
-//function makes it into the mainstream libclang release.
-QStringList templateTypeArguments(CXCursor cursor)
-{
-    QStringList types;
-    QString tStr = ClangString(clang_getTypeSpelling(clang_getCursorType(cursor))).toString();
-    ParamIterator iter(QStringLiteral("<>"), tStr);
-
-    while (iter) {
-        types.append(*iter);
-        ++iter;
-    }
-
-    return types;
-}
-
 CXChildVisitResult templateParamsHelper(CXCursor cursor, CXCursor /*parent*/, CXClientData data)
 {
     CXCursorKind kind = clang_getCursorKind(cursor);
@@ -120,13 +104,18 @@ FuncOverrideInfo processCXXMethod(CXCursor cursor, OverrideInfo* info)
 
 CXChildVisitResult baseClassVisitor(CXCursor cursor, CXCursor /*parent*/, CXClientData data);
 
-void processBaseClass(CXCursor cursor, FunctionOverrideList* functionList)
+void processBaseClass(CXCursor cursor, CXCursor parent, FunctionOverrideList* functionList)
 {
     QStringList concrete;
     CXCursor ref = clang_getCursorReferenced(cursor);
+
+    if (clang_equalCursors(ref, parent)) {
+        return;
+    }
+
     CXCursor isTemplate = clang_getSpecializedCursorTemplate(ref);
     if (!clang_Cursor_isNull(isTemplate)) {
-        concrete = templateTypeArguments(ref);
+        concrete = ClangUtils::templateArgumentTypes(ref);
         ref = isTemplate;
     }
 
@@ -134,7 +123,7 @@ void processBaseClass(CXCursor cursor, FunctionOverrideList* functionList)
     clang_visitChildren(ref, baseClassVisitor, &info);
 }
 
-CXChildVisitResult baseClassVisitor(CXCursor cursor, CXCursor /*parent*/, CXClientData data)
+CXChildVisitResult baseClassVisitor(CXCursor cursor, CXCursor parent, CXClientData data)
 {
     QString templateParam;
     OverrideInfo* info = static_cast<OverrideInfo*>(data);
@@ -149,7 +138,7 @@ CXChildVisitResult baseClassVisitor(CXCursor cursor, CXCursor /*parent*/, CXClie
         }
         return CXChildVisit_Continue;
     case CXCursor_CXXBaseSpecifier:
-        processBaseClass(cursor, info->functions);
+        processBaseClass(cursor, parent, info->functions);
         return CXChildVisit_Continue;
     case CXCursor_CXXMethod:
         if (clang_CXXMethod_isVirtual(cursor)) {
@@ -161,11 +150,11 @@ CXChildVisitResult baseClassVisitor(CXCursor cursor, CXCursor /*parent*/, CXClie
     }
 }
 
-CXChildVisitResult findBaseVisitor(CXCursor cursor, CXCursor /*parent*/, CXClientData data)
+CXChildVisitResult findBaseVisitor(CXCursor cursor, CXCursor parent, CXClientData data)
 {
     auto cursorKind = clang_getCursorKind(cursor);
     if (cursorKind == CXCursor_CXXBaseSpecifier) {
-        processBaseClass(cursor, static_cast<FunctionOverrideList*>(data));
+        processBaseClass(cursor, parent, static_cast<FunctionOverrideList*>(data));
     } else if (cursorKind == CXCursor_CXXMethod)   {
         if (!clang_CXXMethod_isVirtual(cursor)) {
             return CXChildVisit_Continue;
@@ -244,13 +233,9 @@ CXChildVisitResult declVisitor(CXCursor cursor, CXCursor parent, CXClientData d)
             if (kind == CXCursor_ClassTemplate || kind == CXCursor_ClassTemplatePartialSpecialization) {
                 //If we're at a template, we need to construct the template<typename T1, typename T2>
                 //which goes at the front of the prototype
-                QStringList templateTypes = templateParams(kind == CXCursor_ClassTemplate ? cursor : clang_getSpecializedCursorTemplate(cursor));
+                const QStringList templateTypes = templateParams(kind == CXCursor_ClassTemplate ? cursor : clang_getSpecializedCursorTemplate(cursor));
 
-                templatePrefix = QLatin1String("template<");
-                for (int i = 0; i < templateTypes.count(); i++) {
-                    templatePrefix = templatePrefix + QLatin1String((i > 0) ? ", " : "") + QLatin1String("typename ") + templateTypes.at(i);
-                }
-                templatePrefix = templatePrefix + QLatin1String("> ");
+                templatePrefix = QLatin1String("template<") + templateTypes.join(QStringLiteral(", typename ")) + QLatin1String("> ");
             }
         }
 
@@ -274,6 +259,11 @@ CXChildVisitResult declVisitor(CXCursor cursor, CXCursor parent, CXClientData d)
         return CXChildVisit_Continue;
     }
 
+    // don't try to implement pure virtual functions
+    if (clang_CXXMethod_isPureVirtual(cursor)) {
+        return CXChildVisit_Continue;
+    }
+
     CXCursor origin = data->origin;
 
     //Don't try to redefine class/structure/union members
@@ -286,12 +276,10 @@ CXChildVisitResult declVisitor(CXCursor cursor, CXCursor parent, CXClientData d)
         return CXChildVisit_Continue;
     }
 
-    if (isQtMocFunction(cursor)) {
+    if (isQtMocFunction(cursor) || ClangUtils::specialQtAttributes(cursor) == ClangUtils::QtSignalAttribute) {
         return CXChildVisit_Continue;
     }
 
-    //TODO Add support for pure virtual functions
-
     const auto scope = ClangUtils::getScope(cursor, data->origin);
     QString signature = ClangUtils::getCursorSignature(cursor, scope);
 
@@ -311,7 +299,7 @@ CXChildVisitResult declVisitor(CXCursor cursor, CXCursor parent, CXClientData d)
         DUChainReadLocker lock;
         top = DUChain::self()->chainForDocument(ClangString(clang_getFileName(file)).toIndexed());
     }
-    DeclarationPointer declaration = ClangHelpers::findDeclaration(clang_getCursorLocation(cursor), top);
+    DeclarationPointer declaration = ClangHelpers::findDeclaration(clang_getCursorLocation(cursor), QualifiedIdentifier(), top);
     data->prototypes->append(FuncImplementInfo{kind == CXCursor_Constructor, kind == CXCursor_Destructor,
                                                data->templatePrefix, returnType, rest, declaration});
 
@@ -346,6 +334,8 @@ void CompletionHelper::computeCompletions(const ParseSession& session, CXFile fi
     CXCursor currentCursor = clang_getCursor(unit, location);
     if (clang_getCursorKind(currentCursor) == CXCursor_NoDeclFound) {
         currentCursor = topCursor;
+    } else if (KTextEditor::Cursor(ClangLocation(clang_getCursorLocation(currentCursor))) >= ClangLocation(location)) {
+        currentCursor = clang_getCursorLexicalParent(currentCursor);
     }
 
     clang_visitChildren(currentCursor, findBaseVisitor, &m_overrides);
