@@ -23,9 +23,8 @@
 #include <interfaces/ilanguagecontroller.h>
 #include <language/assistant/renameaction.h>
 #include <language/duchain/duchainutils.h>
-#include <language/backgroundparser/backgroundparser.h>
-#include <language/backgroundparser/parsejob.h>
 #include <language/duchain/functiondefinition.h>
+#include <language/duchain/classfunctiondeclaration.h>
 #include <language/duchain/types/functiontype.h>
 
 #include <KTextEditor/Document>
@@ -48,6 +47,12 @@ Declaration *getDeclarationAtCursor(const KTextEditor::Cursor &cursor, const QUr
     return context->type() == DUContext::Function ? context->owner() : nullptr;
 }
 
+bool isConstructor(const Declaration *functionDecl)
+{
+    auto classFun = dynamic_cast<const ClassFunctionDeclaration*>(functionDecl);
+    return classFun && classFun->isConstructor();
+}
+
 Signature getDeclarationSignature(const Declaration *functionDecl, const DUContext *functionCtxt, bool includeDefaults)
 {
     ENSURE_CHAIN_READ_LOCKED
@@ -61,9 +66,10 @@ Signature getDeclarationSignature(const Declaration *functionDecl, const DUConte
     }
     signature.isConst = functionDecl->abstractType() && functionDecl->abstractType()->modifiers() & AbstractType::ConstModifier;
 
-    FunctionType::Ptr funType = functionDecl->type<FunctionType>();
-    if (funType) {
-        signature.returnType = funType->returnType()->indexed();
+    if (!isConstructor(functionDecl)) {
+        if (auto funType = functionDecl->type<FunctionType>()) {
+            signature.returnType = IndexedType(funType->returnType());
+        }
     }
     return signature;
 }
@@ -72,13 +78,11 @@ Signature getDeclarationSignature(const Declaration *functionDecl, const DUConte
 AdaptSignatureAssistant::AdaptSignatureAssistant(ILanguageSupport* supportedLanguage)
     : StaticAssistant(supportedLanguage)
 {
-    connect(ICore::self()->languageController()->backgroundParser(), &BackgroundParser::parseJobFinished,
-            this, &AdaptSignatureAssistant::parseJobFinished);
 }
 
 QString AdaptSignatureAssistant::title() const
 {
-    return tr("Adapt Signature");
+    return i18n("Adapt Signature");
 }
 
 void AdaptSignatureAssistant::reset()
@@ -92,21 +96,21 @@ void AdaptSignatureAssistant::reset()
     m_otherSideTopContext = {};
     m_otherSideContext = {};
     m_oldSignature = {};
-    m_document = {};
+    m_document = nullptr;
     m_view.clear();
 }
 
-void AdaptSignatureAssistant::textChanged(KTextEditor::View* view, const KTextEditor::Range& invocationRange, const QString& removedText)
+void AdaptSignatureAssistant::textChanged(KTextEditor::Document* doc, const KTextEditor::Range& invocationRange, const QString& removedText)
 {
     reset();
 
-    m_view = view;
+    m_document = doc;
+    m_lastEditPosition = invocationRange.end();
 
     KTextEditor::Range sigAssistRange = invocationRange;
     if (!removedText.isEmpty()) {
         sigAssistRange.setRange(sigAssistRange.start(), sigAssistRange.start());
     }
-    m_document = view->document()->url();
 
     DUChainReadLocker lock(DUChain::lock(), 300);
     if (!lock.locked()) {
@@ -114,8 +118,9 @@ void AdaptSignatureAssistant::textChanged(KTextEditor::View* view, const KTextEd
         return;
     }
     KTextEditor::Range simpleInvocationRange = KTextEditor::Range(sigAssistRange);
-    Declaration* funDecl = getDeclarationAtCursor(simpleInvocationRange.start(), m_document);
+    Declaration* funDecl = getDeclarationAtCursor(simpleInvocationRange.start(), m_document->url());
     if (!funDecl || !funDecl->type<FunctionType>()) {
+        clangDebug() << "No function at cursor";
         return;
     }
     /*
@@ -138,10 +143,12 @@ void AdaptSignatureAssistant::textChanged(KTextEditor::View* view, const KTextEd
         otherSide = definition;
     }
     if (!otherSide) {
+        clangDebug() << "no other side for signature found";
         return;
     }
     m_otherSideContext = DUContextPointer(DUChainUtils::getFunctionContext(otherSide));
     if (!m_otherSideContext) {
+        clangDebug() << "no context for other side found";
         return;
     }
     m_declarationName = funDecl->identifier();
@@ -155,7 +162,7 @@ void AdaptSignatureAssistant::textChanged(KTextEditor::View* view, const KTextEd
 
 bool AdaptSignatureAssistant::isUseful() const
 {
-    return !m_declarationName.isEmpty() && m_otherSideId.isValid();
+    return !m_declarationName.isEmpty() && m_otherSideId.isValid() && !actions().isEmpty();
 }
 
 bool AdaptSignatureAssistant::getSignatureChanges(const Signature& newSignature, QList<int>& oldPositions) const
@@ -245,18 +252,18 @@ QList<RenameAction*> AdaptSignatureAssistant::getRenameActions(const Signature &
     return renameActions;
 }
 
-void AdaptSignatureAssistant::parseJobFinished(KDevelop::ParseJob* job)
+void AdaptSignatureAssistant::updateReady(const KDevelop::IndexedString& document, const KDevelop::ReferencedTopDUContext& top)
 {
-    if (job->document().toUrl() != m_document || !m_view) {
+    if (!top || !m_document || document.toUrl() != m_document->url() || top->url() != IndexedString(m_document->url())) {
         return;
     }
     clearActions();
 
     DUChainReadLocker lock;
 
-    Declaration *functionDecl = getDeclarationAtCursor(KTextEditor::Cursor(m_view.data()->cursorPosition()), m_document);
+    Declaration *functionDecl = getDeclarationAtCursor(m_lastEditPosition, m_document->url());
     if (!functionDecl || functionDecl->identifier() != m_declarationName) {
-        clangDebug() << "No function found at" << m_document << m_view.data()->cursorPosition();
+        clangDebug() << "No function found at" << m_document->url() << m_lastEditPosition;
         return;
     }
     DUContext *functionCtxt = DUChainUtils::getFunctionContext(functionDecl);
@@ -280,6 +287,7 @@ void AdaptSignatureAssistant::parseJobFinished(KDevelop::ParseJob* job)
     QList<int> oldPositions;
     if (!getSignatureChanges(newSignature, oldPositions)) {
         reset();
+        clangDebug() << "no changes to signature";
         return; //No changes to signature
     }
     QList<RenameAction*> renameActions;
@@ -297,4 +305,16 @@ void AdaptSignatureAssistant::parseJobFinished(KDevelop::ParseJob* job)
     emit actionsChanged();
 }
 
+KTextEditor::Range AdaptSignatureAssistant::displayRange() const
+{
+    if (!m_document) {
+        return {};
+    }
+
+    auto s = m_lastEditPosition;
+    KTextEditor::Range ran = {s.line(), 0, s.line(), m_document->lineLength(s.line())};
+    qDebug() << "display range:" << ran;
+    return ran;
+}
+
 #include "moc_adaptsignatureassistant.cpp"
