@@ -39,6 +39,7 @@
 #include <language/duchain/classdeclaration.h>
 #include <language/duchain/stringhelpers.h>
 #include <language/duchain/duchainutils.h>
+#include <language/duchain/problem.h>
 
 #include <language/duchain/types/pointertype.h>
 #include <language/duchain/types/arraytype.h>
@@ -246,8 +247,9 @@ struct DeclType<CK, isDefinition, isInClass,
 //BEGIN CurrentContext
 struct CurrentContext
 {
-    CurrentContext(DUContext* context)
+    CurrentContext(DUContext* context, QSet<DUContext*> keepAliveContexts)
         : context(context)
+        , keepAliveContexts(keepAliveContexts)
     {
         DUChainReadLocker lock;
         previousChildContexts = context->childContexts();
@@ -257,7 +259,11 @@ struct CurrentContext
     ~CurrentContext()
     {
         DUChainWriteLocker lock;
-        qDeleteAll(previousChildContexts);
+        foreach (auto childContext, previousChildContexts) {
+            if (!keepAliveContexts.contains(childContext)) {
+                delete childContext;
+            }
+        }
         qDeleteAll(previousChildDeclarations);
         if (resortChildContexts) {
             context->resortChildContexts();
@@ -268,9 +274,11 @@ struct CurrentContext
     }
 
     DUContext* context;
-    // when updatig, this contains child contexts of the current parent context
+    // when updating, this contains child contexts of the current parent context
     QVector<DUContext*> previousChildContexts;
-    // when updatig, this contains child declarations of the current parent context
+    // when updating, this contains contexts that must not be deleted
+    QSet<DUContext*> keepAliveContexts;
+    // when updating, this contains child declarations of the current parent context
     QVector<Declaration*> previousChildDeclarations;
 
     bool resortChildContexts = false;
@@ -430,9 +438,10 @@ struct Visitor
         if (Type == DUContext::Other || Type == DUContext::Function)
             context->setInSymbolTable(false);
 
-        if (CK == CXCursor_CXXMethod || CursorKindTraits::isClass(CK)) {
+        if (CK == CXCursor_CXXMethod) {
             CXCursor semParent = clang_getCursorSemanticParent(cursor);
-            if (!clang_Cursor_isNull(semParent)) {
+            // only import the semantic parent if it differs from the lexical parent
+            if (!clang_Cursor_isNull(semParent) && !clang_equalCursors(semParent, clang_getCursorLexicalParent(cursor))) {
                 auto semParentDecl = findDeclaration(semParent);
                 if (semParentDecl) {
                     contextImportDecl(context, semParentDecl);
@@ -477,7 +486,7 @@ struct Visitor
         return ref;
     }
 
-    template<CXTypeKind TK, EnableIf<TK == CXType_FunctionProto> = dummy>
+    template<CXTypeKind TK, EnableIf<TK == CXType_FunctionProto || TK == CXType_FunctionNoProto> = dummy>
     AbstractType *createType(CXType type, CXCursor parent)
     {
         auto func = new FunctionType;
@@ -514,7 +523,11 @@ struct Visitor
         }
 
         auto t = new StructureType;
-        t->setDeclaration(decl.data());
+        if (decl) {
+            t->setDeclaration(decl.data());
+        } else {    // fallback, at least give the spelling to the user
+            t->setDeclarationId(DeclarationId(IndexedQualifiedIdentifier(QualifiedIdentifier(ClangString(clang_getTypeSpelling(type)).toString()))));
+        }
         return t;
     }
 
@@ -555,8 +568,8 @@ struct Visitor
     AbstractType *createType(CXType type, CXCursor parent)
     {
         auto numTA = clang_Type_getNumTemplateArguments(type);
-        if (numTA != -1) {
-            // This is a class template specialization.
+        // TODO: We should really expose more types to libclang!
+        if (numTA != -1 && ClangString(clang_getTypeSpelling(type)).toString().contains(QLatin1Char('<'))) {
             return createClassTemplateSpecializationType(type);
         }
 
@@ -633,6 +646,15 @@ struct Visitor
     }
 #endif
 
+#if CINDEX_VERSION_MINOR >= 34
+    template<CXTypeKind TK, EnableIf<TK == CXType_Elaborated> = dummy>
+    AbstractType *createType(CXType type, CXCursor parent)
+    {
+        auto underyingType = clang_Type_getNamedType(type);
+        return makeType(underyingType, parent);
+    }
+#endif
+
     /// @param declaration an optional declaration that will be associated with created type
     AbstractType* createClassTemplateSpecializationType(CXType type, const DeclarationPointer declaration = {})
     {
@@ -680,7 +702,13 @@ struct Visitor
         auto decl = declaration ? declaration : findDeclaration(typeDecl);
 
         DUChainReadLocker lock;
-        cst->setDeclaration(decl.data());
+        if (decl) {
+            cst->setDeclaration(decl.data());
+        } else { // fallback, at least give the spelling to the user
+            Identifier id(tStr);
+            id.clearTemplateIdentifiers();
+            cst->setDeclarationId(DeclarationId(IndexedQualifiedIdentifier(QualifiedIdentifier(id))));
+        }
 
         return cst;
     }
@@ -961,10 +989,16 @@ void Visitor::setDeclData(CXCursor cursor, ClassMemberDeclaration *decl) const
             auto type = clang_getCursorType(cursor);
             auto sizeOf = clang_Type_getSizeOf(type);
             auto alignedTo = clang_Type_getAlignOf(type);
+            const auto byteOffset = offset / 8;
+            const auto bitOffset = offset % 8;
+            const QString byteOffsetStr = i18np("1 Byte", "%1 Bytes", byteOffset);
+            const QString bitOffsetStr = bitOffset ? i18np("1 Bit", "%1 Bits", bitOffset) : QString();
+            const QString offsetStr = bitOffset ? i18nc("%1: bytes, %2: bits", "%1, %2", byteOffsetStr, bitOffsetStr) : byteOffsetStr;
+
             decl->setComment(decl->comment()
-                                + i18n("<br/>offset in parent: %1 Bit<br/>"
-                                    "size: %2 Bytes<br/>"
-                                    "aligned to: %3 Bytes", offset, sizeOf, alignedTo).toUtf8());
+                                + i18n("<p>offset in parent: %1; "
+                                    "size: %2 Bytes; "
+                                    "aligned to: %3 Bytes</p>", offsetStr, sizeOf, alignedTo).toUtf8());
         }
     }
 #endif
@@ -1002,8 +1036,8 @@ void Visitor::setDeclData(CXCursor cursor, ClassDeclaration* decl) const
         auto alignOf = clang_Type_getAlignOf(type);
         if (sizeOf >= 0 && alignOf >= 0) {
             decl->setComment(decl->comment()
-                                + i18n("<br/>size: %1 Bytes<br/>"
-                                    "aligned to: %2 Bytes", sizeOf, alignOf).toUtf8());
+                                + i18n("<p>size: %1 Bytes; "
+                                    "aligned to: %2 Bytes</p>", sizeOf, alignOf).toUtf8());
         }
     }
 }
@@ -1031,6 +1065,10 @@ void Visitor::setDeclData(CXCursor cursor, ClassFunctionDeclaration* decl) const
     decl->setAbstract(clang_CXXMethod_isPureVirtual(cursor));
     decl->setStatic(clang_CXXMethod_isStatic(cursor));
     decl->setVirtual(clang_CXXMethod_isVirtual(cursor));
+
+    const auto qtAttribute = ClangUtils::specialQtAttributes(cursor);
+    decl->setIsSignal(qtAttribute == ClangUtils::QtSignalAttribute);
+    decl->setIsSlot(qtAttribute == ClangUtils::QtSlotAttribute);
 }
 
 template<CXCursorKind CK>
@@ -1051,11 +1089,14 @@ template<CXCursorKind CK>
 void Visitor::setDeclData(CXCursor cursor, NamespaceAliasDeclaration *decl) const
 {
     setDeclData<CK>(cursor, static_cast<Declaration*>(decl));
-    clang_visitChildren(cursor, [] (CXCursor cursor, CXCursor /*parent*/, CXClientData data) -> CXChildVisitResult {
-        Q_ASSERT(clang_getCursorKind(cursor) == CXCursor_NamespaceRef);
-        const auto id = QualifiedIdentifier(ClangString(clang_getCursorSpelling(cursor)).toString());
-        reinterpret_cast<NamespaceAliasDeclaration*>(data)->setImportIdentifier(id);
-        return CXChildVisit_Break;
+    clang_visitChildren(cursor, [] (CXCursor cursor, CXCursor parent, CXClientData data) -> CXChildVisitResult {
+        if (clang_getCursorKind(cursor) == CXCursor_NamespaceRef) {
+            const auto id = QualifiedIdentifier(ClangString(clang_getCursorSpelling(cursor)).toString());
+            reinterpret_cast<NamespaceAliasDeclaration*>(data)->setImportIdentifier(id);
+            return CXChildVisit_Break;
+        } else {
+            return visitCursor(cursor, parent, data);
+        }
     }, decl);
 }
 //END setDeclData
@@ -1082,7 +1123,7 @@ CXChildVisitResult Visitor::buildDeclaration(CXCursor cursor)
         if (isOutOfLine) {
             const QString scope = ClangUtils::getScope(cursor);
             auto context = createContext<CK, DUContext::Helper>(cursor, QualifiedIdentifier(scope));
-            helperContext.reset(new CurrentContext(context));
+            helperContext.reset(new CurrentContext(context, m_parentContext->keepAliveContexts));
         }
     }
 
@@ -1092,7 +1133,7 @@ CXChildVisitResult Visitor::buildDeclaration(CXCursor cursor)
     if (hasContext) {
         auto context = createContext<CK, CursorKindTraits::contextType(CK)>(cursor, QualifiedIdentifier(id));
         createDeclaration<CK, DeclType>(cursor, id, context);
-        CurrentContext newParent(context);
+        CurrentContext newParent(context, m_parentContext->keepAliveContexts);
         PushValue<CurrentContext*> pushCurrent(m_parentContext, &newParent);
         clang_visitChildren(cursor, &visitCursor, this);
         return CXChildVisit_Continue;
@@ -1130,7 +1171,7 @@ CXChildVisitResult Visitor::buildCompoundStatement(CXCursor cursor)
     if (m_parentContext->context->type() == DUContext::Function)
     {
         auto context = createContext<CXCursor_CompoundStmt, DUContext::Other>(cursor);
-        CurrentContext newParent(context);
+        CurrentContext newParent(context, m_parentContext->keepAliveContexts);
         PushValue<CurrentContext*> pushCurrent(m_parentContext, &newParent);
         clang_visitChildren(cursor, &visitCursor, this);
         return CXChildVisit_Continue;
@@ -1188,7 +1229,7 @@ void Visitor::setIdTypeDecl(CXCursor typeCursor, IdentifiedType* idType) const
 
 AbstractType *Visitor::makeType(CXType type, CXCursor parent)
 {
-    #define UseKind(TypeKind) case TypeKind: return dispatchType<TypeKind>(type, parent)
+#define UseKind(TypeKind) case TypeKind: return dispatchType<TypeKind>(type, parent)
     switch (type.kind) {
     UseKind(CXType_Void);
     UseKind(CXType_Bool);
@@ -1210,6 +1251,7 @@ AbstractType *Visitor::makeType(CXType type, CXCursor parent)
     UseKind(CXType_Char16);
     UseKind(CXType_Char32);
     UseKind(CXType_Pointer);
+    UseKind(CXType_BlockPointer);
     UseKind(CXType_MemberPointer);
     UseKind(CXType_ObjCObjectPointer);
     UseKind(CXType_ConstantArray);
@@ -1218,6 +1260,7 @@ AbstractType *Visitor::makeType(CXType type, CXCursor parent)
     UseKind(CXType_DependentSizedArray);
     UseKind(CXType_LValueReference);
     UseKind(CXType_RValueReference);
+    UseKind(CXType_FunctionNoProto);
     UseKind(CXType_FunctionProto);
     UseKind(CXType_Record);
     UseKind(CXType_Enum);
@@ -1234,6 +1277,9 @@ AbstractType *Visitor::makeType(CXType type, CXCursor parent)
     UseKind(CXType_NullPtr);
 #if CINDEX_VERSION_MINOR >= 31
     UseKind(CXType_Auto);
+#endif
+#if CINDEX_VERSION_MINOR >= 34
+    UseKind(CXType_Elaborated);
 #endif
     case CXType_Invalid:
         return nullptr;
@@ -1241,6 +1287,7 @@ AbstractType *Visitor::makeType(CXType type, CXCursor parent)
         qCWarning(KDEV_CLANG) << "Unhandled type:" << type.kind << clang_getTypeSpelling(type);
         return nullptr;
     }
+#undef UseKind
 }
 
 RangeInRevision rangeInRevisionForUse(CXCursor cursor, CXCursorKind referencedCursorKind, CXSourceRange useRange, const QSet<unsigned int>& macroExpansionLocations)
@@ -1294,11 +1341,33 @@ Visitor::Visitor(CXTranslationUnit tu, CXFile file,
     , m_update(update)
 {
     CXCursor tuCursor = clang_getTranslationUnitCursor(tu);
-    CurrentContext parent(includes[file]);
+    auto top = includes[file];
+
+    // when updating, this contains child contexts that should be kept alive
+    // even when they are not part of the AST anymore
+    // this is required for some assistants, such as the signature assistant
+    QSet<DUContext*> keepAliveContexts;
+    {
+        DUChainReadLocker lock;
+        foreach (const auto& problem, top->problems()) {
+            const auto& desc = problem->description();
+            if (desc.startsWith(QLatin1String("Return type of out-of-line definition of '"))
+                && desc.endsWith(QLatin1String("' differs from that in the declaration"))) {
+                auto ctx = top->findContextAt(problem->range().start);
+                // keep the context and its parents alive
+                // this also keeps declarations in this context alive
+                while (ctx) {
+                    keepAliveContexts << ctx;
+                    ctx = ctx->parentContext();
+                }
+            }
+        }
+    }
+
+    CurrentContext parent(top, keepAliveContexts);
     m_parentContext = &parent;
     clang_visitChildren(tuCursor, &visitCursor, this);
 
-    TopDUContext *top = m_parentContext->context->topContext();
     if (m_update) {
         DUChainWriteLocker lock;
         top->deleteUsesRecursively();
@@ -1437,4 +1506,4 @@ void visit(CXTranslationUnit tu, CXFile file, const IncludeFileContexts& include
     Visitor visitor(tu, file, includes, update);
 }
 
-}
\ No newline at end of file
+}
