@@ -93,20 +93,32 @@ Imports ClangHelpers::tuImports(CXTranslationUnit tu)
     return imports;
 }
 
+bool importLocationLessThan(const Import& lhs, const Import& rhs)
+{
+    return lhs.location.line < rhs.location.line;
+}
+
 ReferencedTopDUContext ClangHelpers::buildDUChain(CXFile file, const Imports& imports, const ParseSession& session,
                                                   TopDUContext::Features features, IncludeFileContexts& includedFiles,
-                                                  ClangIndex* index)
+                                                  ClangIndex* index, const std::function<bool()>& abortFunction)
 {
     if (includedFiles.contains(file)) {
         return {};
     }
 
+    if (abortFunction && abortFunction()) {
+        return {};
+    }
+
     // prevent recursion
     includedFiles.insert(file, {});
 
-    // ensure DUChain for imports are build properly
-    foreach(const auto& import, imports.values(file)) {
-        buildDUChain(import.file, imports, session, features, includedFiles, index);
+    // ensure DUChain for imports are built properly, and in correct order
+    QList<Import> sortedImports = imports.values(file);
+    std::sort(sortedImports.begin(), sortedImports.end(), importLocationLessThan);
+
+    foreach(const auto& import, sortedImports) {
+        buildDUChain(import.file, imports, session, features, includedFiles, index, abortFunction);
     }
 
     const IndexedString path(QDir(ClangString(clang_getFileName(file)).toString()).canonicalPath());
@@ -133,7 +145,16 @@ ReferencedTopDUContext ClangHelpers::buildDUChain(CXFile file, const Imports& im
         if (update) {
             auto envFile = ClangParsingEnvironmentFile::Ptr(dynamic_cast<ClangParsingEnvironmentFile*>(context->parsingEnvironmentFile().data()));
             Q_ASSERT(envFile);
-            if (!envFile->needsUpdate(&environment) && envFile->featuresSatisfied(features)) {
+            if (!envFile)
+                return context;
+
+            /* NOTE: When we are here, then either the translation unit or one of its headers was changed.
+             *       Thus we must always update the translation unit to propagate the change(s).
+             *       See also: https://bugs.kde.org/show_bug.cgi?id=356327
+             *       This assumes that headers are independent, we may need to improve that in the future
+             *       and also update header files more often when other files included therein got updated.
+             */
+            if (path != environment.translationUnitUrl() && !envFile->needsUpdate(&environment) && envFile->featuresSatisfied(features)) {
                 return context;
             } else {
                 //TODO: don't attempt to update if this environment is worse quality than the outdated one
@@ -148,8 +169,7 @@ ReferencedTopDUContext ClangHelpers::buildDUChain(CXFile file, const Imports& im
         }
         context->setFeatures(features);
 
-        foreach(const auto& import, imports.values(file)) {
-            Q_ASSERT(includedFiles.contains(import.file));
+        foreach(const auto& import, sortedImports) {
             auto ctx = includedFiles.value(import.file);
             if (!ctx) {
                 // happens for cyclic imports
@@ -168,10 +188,12 @@ ReferencedTopDUContext ClangHelpers::buildDUChain(CXFile file, const Imports& im
 
     Builder::visit(session.unit(), file, includedFiles, update);
 
+    DUChain::self()->emitUpdateReady(path, context);
+
     return context;
 }
 
-DeclarationPointer ClangHelpers::findDeclaration(CXSourceLocation location, const ReferencedTopDUContext& top)
+DeclarationPointer ClangHelpers::findDeclaration(CXSourceLocation location, QualifiedIdentifier id, const ReferencedTopDUContext& top)
 {
     if (!top) {
         // may happen for cyclic includes
@@ -180,6 +202,24 @@ DeclarationPointer ClangHelpers::findDeclaration(CXSourceLocation location, cons
 
     auto cursor = CursorInRevision(ClangLocation(location));
     DUChainReadLocker lock;
+
+    QList<Declaration*> decls;
+    if (!id.isEmpty())
+    {
+        decls = top->findDeclarations(id);
+        foreach (Declaration* decl, decls)
+        {
+            if (decl->range().contains(cursor) ||
+                (decl->range().isEmpty() && decl->range().start == cursor))
+            {
+                return DeclarationPointer(decl);
+            }
+        }
+    }
+
+    // there was no match based on the IDs, try the classical
+    // range based search (very slow)
+
     Q_ASSERT(top);
     if (DUContext *local = top->findContextAt(cursor)) {
         if (local->owner() && local->owner()->range().contains(cursor)) {
@@ -199,7 +239,22 @@ DeclarationPointer ClangHelpers::findDeclaration(CXCursor cursor, const IncludeF
         return {};
     }
 
-    return findDeclaration(location, includes.value(file));
+    // build a qualified identifier by following the chain of semantic parents
+    QList<Identifier> ids;
+    CXCursor currentCursor = cursor;
+    while (currentCursor.kind != CXCursor_TranslationUnit &&
+           currentCursor.kind != CXCursor_InvalidFile)
+    {
+        ids << Identifier(ClangString(clang_getCursorSpelling(currentCursor)).toString());
+        currentCursor = clang_getCursorSemanticParent(currentCursor);
+    }
+    QualifiedIdentifier qid;
+    for (int i = ids.size()-1; i >= 0; --i)
+    {
+        qid.push(ids[i]);
+    }
+
+    return findDeclaration(location, qid, includes.value(file));
 }
 
 DeclarationPointer ClangHelpers::findDeclaration(CXType type, const IncludeFileContexts& includes)
@@ -277,14 +332,14 @@ QStringList ClangHelpers::sourceExtensions()
 
 bool ClangHelpers::isSource(const QString& path)
 {
-    auto extensions = sourceExtensions();
+    const auto& extensions = sourceExtensions();
     return std::any_of(extensions.constBegin(), extensions.constEnd(),
                        [&](const QString& ext) { return path.endsWith(ext); });
 }
 
 bool ClangHelpers::isHeader(const QString& path)
 {
-    auto extensions = headerExtensions();
+    const auto& extensions = headerExtensions();
     return std::any_of(extensions.constBegin(), extensions.constEnd(),
                        [&](const QString& ext) { return path.endsWith(ext); });
 }
