@@ -29,6 +29,8 @@
 #include <util/clangtypes.h>
 #include <util/clangdebug.h>
 #include <language/backgroundparser/urlparselock.h>
+#include <language/duchain/duchainlock.h>
+#include <language/duchain/duchain.h>
 
 #include <clang-c/Index.h>
 
@@ -38,6 +40,11 @@ ClangIndex::ClangIndex()
     // NOTE: We don't exclude PCH declarations. That way we could retrieve imports manually, as clang_getInclusions returns nothing on reparse with CXTranslationUnit_PrecompiledPreamble flag.
     : m_index(clang_createIndex(0 /*Exclude PCH Decls*/, qEnvironmentVariableIsSet("KDEV_CLANG_DISPLAY_DIAGS") /*Display diags*/))
 {
+    // demote the priority of the clang parse threads to reduce potential UI lockups
+    // but the code completion threads still retain their normal priority to return
+    // the results as quickly as possible
+    clang_CXIndex_setGlobalOptions(m_index, clang_CXIndex_getGlobalOptions(m_index)
+        | CXGlobalOpt_ThreadBackgroundPriorityForIndexing);
 }
 
 CXIndex ClangIndex::index() const
@@ -89,6 +96,34 @@ IndexedString ClangIndex::translationUnitForUrl(const IndexedString& url)
             return tu.value();
         }
     }
+    // if no explicit pin data is available, follow back the duchain import chain
+    {
+        KDevelop::DUChainReadLocker lock;
+        TopDUContext* top = DUChain::self()->chainForDocument(url);
+        if (top) {
+            TopDUContext* tuTop = top;
+            QSet<TopDUContext*> visited;
+            while(true) {
+                visited.insert(tuTop);
+                TopDUContext* next = NULL;
+                auto importers = tuTop->indexedImporters();
+                foreach(IndexedDUContext ctx, importers) {
+                    if (ctx.data()) {
+                        next = ctx.data()->topContext();
+                        break;
+                    }
+                }
+                if (!next || visited.contains(next)) {
+                    break;
+                }
+                tuTop = next;
+            }
+            if (tuTop != top) {
+                return tuTop->url();
+            }
+        }
+    }
+
     // otherwise, fallback to a simple buddy search for headers
     if (ClangHelpers::isHeader(url.str())) {
         foreach(const QUrl& buddy, DocumentFinderHelpers::getPotentialBuddies(url.toUrl(), false)) {
