@@ -19,6 +19,8 @@
 
 #include "test_assistants.h"
 
+#include "codegen/clangrefactoring.h"
+
 #include <QtTest/QtTest>
 #include <QTemporaryDir>
 
@@ -27,6 +29,7 @@
 
 #include <tests/autotestshell.h>
 #include <tests/testcore.h>
+#include <tests/testfile.h>
 
 #include <util/foregroundlock.h>
 
@@ -38,8 +41,10 @@
 #include <language/assistant/staticassistant.h>
 #include <language/assistant/staticassistantsmanager.h>
 #include <language/assistant/renameaction.h>
+#include <language/assistant/renameassistant.h>
 #include <language/backgroundparser/backgroundparser.h>
 #include <language/duchain/duchain.h>
+#include <language/duchain/duchainlock.h>
 #include <language/duchain/duchainutils.h>
 #include <language/codegen/coderepresentation.h>
 
@@ -55,13 +60,14 @@ StaticAssistantsManager *staticAssistantsManager() { return Core::self()->langua
 
 void TestAssistants::initTestCase()
 {
-    QLoggingCategory::setFilterRules(QStringLiteral(R"(
-        *.debug=false
-        default.debug=true
-        kdevelop.plugins.clang.debug=true
-    )"));
+    QLoggingCategory::setFilterRules(QStringLiteral(
+        "*.debug=false\n"
+        "default.debug=true\n"
+        "kdevelop.plugins.clang.debug=true\n"
+    ));
     QVERIFY(qputenv("KDEV_DISABLE_PLUGINS", "kdevcppsupport"));
-    AutoTestShell::init({QStringLiteral("kdevclangsupport")});
+    QVERIFY(qputenv("KDEV_CLANG_DISPLAY_DIAGS", "1"));
+    AutoTestShell::init({QStringLiteral("kdevclangsupport"), QStringLiteral("kdevproblemreporter")});
     TestCore::initialize();
     DUChain::self()->disablePersistentStorage();
     Core::self()->languageController()->backgroundParser()->setDelay(0);
@@ -125,8 +131,6 @@ public:
         Core::self()->documentController()->documentForUrl(m_cppDocument.url)->textDocument();
         Core::self()->documentController()->documentForUrl(m_cppDocument.url)->close(KDevelop::IDocument::Discard);
         Core::self()->documentController()->documentForUrl(m_headerDocument.url)->close(KDevelop::IDocument::Discard);
-
-        staticAssistantsManager()->hideAssistant();
     }
 
     void changeDocument(TestDoc which, Range where, const QString& what, bool waitForUpdate = false)
@@ -224,7 +228,6 @@ struct StateChange
 Q_DECLARE_METATYPE(StateChange)
 Q_DECLARE_METATYPE(QList<StateChange>)
 
-
 void TestAssistants::testRenameAssistant_data()
 {
     QTest::addColumn<QString>("fileContents");
@@ -253,17 +256,6 @@ void TestAssistants::testRenameAssistant_data()
         << (QList<StateChange>() << StateChange(Testbed::CppDoc, Range(0,12,0,13), "u", "u"))
         << "int foo(int u)\n { u = 0; return u; }";
 
-    QTest::newRow("Letter-by-Letter")
-        << "int foo(int i)\n { i = 0; return i; }"
-        << "i"
-        << (QList<StateChange>()
-            << StateChange(Testbed::CppDoc, Range(0,12,0,13), "", "")
-            << StateChange(Testbed::CppDoc, Range(0,12,0,12), "a", "a")
-            << StateChange(Testbed::CppDoc, Range(0,13,0,13), "b", "ab")
-            << StateChange(Testbed::CppDoc, Range(0,14,0,14), "c", "abc")
-        )
-        << "int foo(int abc)\n { abc = 0; return abc; }";
-
     QTest::newRow("Paste Replace")
         << "int foo(int abg)\n { abg = 0; return abg; }"
         << "abg"
@@ -276,6 +268,15 @@ void TestAssistants::testRenameAssistant_data()
         << (QList<StateChange>() << StateChange(Testbed::CppDoc, Range(0,14,0,14), "cdef", "abcdefg"))
         << "int foo(int abcdefg)\n { abcdefg = 0; return abcdefg; }";
 
+    QTest::newRow("Letter-by-Letter Prepend")
+        << "int foo(int i)\n { i = 0; return i; }"
+        << "i"
+        << (QList<StateChange>()
+            << StateChange(Testbed::CppDoc, Range(0,12,0,12), "a", "ai")
+            << StateChange(Testbed::CppDoc, Range(0,13,0,13), "b", "abi")
+            << StateChange(Testbed::CppDoc, Range(0,14,0,14), "c", "abci")
+        )
+        << "int foo(int abci)\n { abci = 0; return abci; }";
     QTest::newRow("Letter-by-Letter Insert")
         << "int foo(int abg)\n { abg = 0; return abg; }"
         << "abg"
@@ -288,27 +289,55 @@ void TestAssistants::testRenameAssistant_data()
         << "int foo(int abcdefg)\n { abcdefg = 0; return abcdefg; }";
 }
 
+ProblemPointer findStaticAssistantProblem(const QVector<ProblemPointer>& problems)
+{
+    const auto renameProblemIt = std::find_if(problems.cbegin(), problems.cend(), [](const ProblemPointer& p) {
+        return dynamic_cast<const StaticAssistantProblem*>(p.constData());
+    });
+    if (renameProblemIt != problems.cend())
+        return *renameProblemIt;
+
+    return {};
+}
+
 void TestAssistants::testRenameAssistant()
 {
     QFETCH(QString, fileContents);
     Testbed testbed("", fileContents);
 
+    const auto document = testbed.document(Testbed::CppDoc);
+    QVERIFY(document);
+
+    QExplicitlySharedDataPointer<IAssistant> assistant;
+
     QFETCH(QString, oldDeclarationName);
     QFETCH(QList<StateChange>, stateChanges);
     foreach(StateChange stateChange, stateChanges)
     {
-        testbed.changeDocument(Testbed::CppDoc, stateChange.range, stateChange.newText);
+        testbed.changeDocument(Testbed::CppDoc, stateChange.range, stateChange.newText, true);
+
+        DUChainReadLocker lock;
+
+        auto topCtx = DUChain::self()->chainForDocument(document->url());
+        QVERIFY(topCtx);
+
+        const auto problem = findStaticAssistantProblem(DUChainUtils::allProblemsForContext(topCtx));
+        if (problem)
+            assistant = problem->solutionAssistant();
+
         if (stateChange.result.isEmpty()) {
-            QVERIFY(!staticAssistantsManager()->activeAssistant() || !staticAssistantsManager()->activeAssistant()->actions().size());
+            QVERIFY(!assistant || !assistant->actions().size());
         } else {
-            QVERIFY(staticAssistantsManager()->activeAssistant() && staticAssistantsManager()->activeAssistant()->actions().size());
-            RenameAction *r = qobject_cast<RenameAction*>(staticAssistantsManager()->activeAssistant()->actions().first().data());
+            qWarning() << assistant.data() << stateChange.result;
+            QVERIFY(assistant && assistant->actions().size());
+            RenameAction *r = qobject_cast<RenameAction*>(assistant->actions().first().data());
             QCOMPARE(r->oldDeclarationName(), oldDeclarationName);
             QCOMPARE(r->newDeclarationName(), stateChange.result);
         }
     }
-    if (staticAssistantsManager()->activeAssistant() && staticAssistantsManager()->activeAssistant()->actions().size()) {
-            staticAssistantsManager()->activeAssistant()->actions().first()->execute();
+
+    if (assistant && assistant->actions().size()) {
+        assistant->actions().first()->execute();
     }
     QFETCH(QString, finalFileContents);
     QCOMPARE(testbed.documentText(Testbed::CppDoc), finalFileContents);
@@ -317,16 +346,29 @@ void TestAssistants::testRenameAssistant()
 void TestAssistants::testRenameAssistantUndoRename()
 {
     Testbed testbed("", "int foo(int i)\n { i = 0; return i; }");
-    testbed.changeDocument(Testbed::CppDoc, Range(0,13,0,13), "d");
-    QVERIFY(staticAssistantsManager()->activeAssistant());
-    QVERIFY(staticAssistantsManager()->activeAssistant()->actions().size() > 0);
-    RenameAction *r = qobject_cast<RenameAction*>(staticAssistantsManager()->activeAssistant()->actions().first().data());
+    testbed.changeDocument(Testbed::CppDoc, Range(0,13,0,13), "d", true);
+
+    const auto document = testbed.document(Testbed::CppDoc);
+    QVERIFY(document);
+
+    DUChainReadLocker lock;
+    auto topCtx = DUChain::self()->chainForDocument(document->url());
+    QVERIFY(topCtx);
+
+    auto firstProblem = findStaticAssistantProblem(DUChainUtils::allProblemsForContext(topCtx));
+    QVERIFY(firstProblem);
+    auto assistant = firstProblem->solutionAssistant();
+    QVERIFY(assistant);
+
+    QVERIFY(assistant->actions().size() > 0);
+    RenameAction *r = qobject_cast<RenameAction*>(assistant->actions().first().data());
+    qWarning() << topCtx->problems() << assistant->actions().first().data() << assistant->actions().size();
     QVERIFY(r);
 
     // now rename the variable back to its original identifier
     testbed.changeDocument(Testbed::CppDoc, Range(0,13,0,14), "");
     // there should be no assistant anymore
-    QVERIFY(!staticAssistantsManager()->activeAssistant());
+    QVERIFY(!assistant || assistant->actions().isEmpty());
 }
 
 const QString SHOULD_ASSIST = "SHOULD_ASSIST"; //An assistant will be visible
@@ -430,6 +472,28 @@ void TestAssistants::testSignatureAssistant_data()
         << (QList<StateChange>() << StateChange(Testbed::HeaderDoc, Range(0, 38, 0, 38), ", char c = 'A'", SHOULD_ASSIST))
         << "class Foo { void bar(bool b, int i = 0, char c = 'A'); };"
         << "void Foo::bar(bool b, int i, char c)\n{}";
+
+    // see https://bugs.kde.org/show_bug.cgi?id=355356
+    QTest::newRow("no_retval_on_ctor")
+        << "class Foo { Foo(); };"
+        << "Foo::Foo()\n{}"
+        << (QList<StateChange>() << StateChange(Testbed::HeaderDoc, Range(0, 16, 0, 16), "char c", SHOULD_ASSIST))
+        << "class Foo { Foo(char c); };"
+        << "Foo::Foo(char c)\n{}";
+
+    // see https://bugs.kde.org/show_bug.cgi?id=298511
+    QTest::newRow("change_return_type_header")
+        << "struct Foo { int bar(); };"
+        << "int Foo::bar()\n{}"
+        << (QList<StateChange>() << StateChange(Testbed::HeaderDoc, Range(0, 13, 0, 16), "char", SHOULD_ASSIST))
+        << "struct Foo { char bar(); };"
+        << "char Foo::bar()\n{}";
+    QTest::newRow("change_return_type_impl")
+        << "struct Foo { int bar(); };"
+        << "int Foo::bar()\n{}"
+        << (QList<StateChange>() << StateChange(Testbed::CppDoc, Range(0, 0, 0, 3), "char", SHOULD_ASSIST))
+        << "struct Foo { char bar(); };"
+        << "char Foo::bar()\n{}";
 }
 
 void TestAssistants::testSignatureAssistant()
@@ -438,20 +502,37 @@ void TestAssistants::testSignatureAssistant()
     QFETCH(QString, cppContents);
     Testbed testbed(headerContents, cppContents);
 
+    QExplicitlySharedDataPointer<IAssistant> assistant;
+
     QFETCH(QList<StateChange>, stateChanges);
     foreach (StateChange stateChange, stateChanges)
     {
         testbed.changeDocument(stateChange.document, stateChange.range, stateChange.newText, true);
 
+        const auto document = testbed.document(stateChange.document);
+        QVERIFY(document);
+
+        DUChainReadLocker lock;
+
+        auto topCtx = DUChain::self()->chainForDocument(document->url());
+        QVERIFY(topCtx);
+
+        const auto problem = findStaticAssistantProblem(DUChainUtils::allProblemsForContext(topCtx));
+        if (problem) {
+            assistant = problem->solutionAssistant();
+        }
+
         if (stateChange.result == SHOULD_ASSIST) {
             QEXPECT_FAIL("change_function_type", "Clang sees that return type of out-of-line definition differs from that in the declaration and won't parse the code...", Abort);
-            QVERIFY(staticAssistantsManager()->activeAssistant() && !staticAssistantsManager()->activeAssistant()->actions().isEmpty());
+            QEXPECT_FAIL("change_return_type_impl", "Clang sees that return type of out-of-line definition differs from that in the declaration and won't include the function's AST and thus we never get updated about the new return type...", Abort);
+            QVERIFY(assistant && !assistant->actions().isEmpty());
         } else {
-            QVERIFY(!staticAssistantsManager()->activeAssistant() || staticAssistantsManager()->activeAssistant()->actions().isEmpty());
+            QVERIFY(!assistant || assistant->actions().isEmpty());
         }
     }
-    if (staticAssistantsManager()->activeAssistant() && !staticAssistantsManager()->activeAssistant()->actions().isEmpty())
-        staticAssistantsManager()->activeAssistant()->actions().first()->execute();
+
+    if (assistant && !assistant->actions().isEmpty())
+        assistant->actions().first()->execute();
 
     QFETCH(QString, finalHeaderContents);
     QFETCH(QString, finalCppContents);
@@ -459,13 +540,13 @@ void TestAssistants::testSignatureAssistant()
     QCOMPARE(testbed.documentText(Testbed::CppDoc), finalCppContents);
 }
 
-enum UnknownDeclarationActions
+enum UnknownDeclarationAction
 {
-    NoUnknownDeclaration = 0x0,
+    NoUnknownDeclarationAction = 0x0,
     ForwardDecls = 0x1,
     MissingInclude = 0x2
 };
-
+Q_DECLARE_FLAGS(UnknownDeclarationActions, UnknownDeclarationAction)
 Q_DECLARE_METATYPE(UnknownDeclarationActions)
 
 void TestAssistants::testUnknownDeclarationAssistant_data()
@@ -476,11 +557,11 @@ void TestAssistants::testUnknownDeclarationAssistant_data()
     QTest::addColumn<UnknownDeclarationActions>("actions");
 
     QTest::newRow("unincluded_struct") << "struct test{};" << "" << "test"
-        << static_cast<UnknownDeclarationActions>(ForwardDecls | MissingInclude);
+        << UnknownDeclarationActions(ForwardDecls | MissingInclude);
     QTest::newRow("forward_declared_struct") << "struct test{};" << "struct test;" << "test *f; f->"
-        << static_cast<UnknownDeclarationActions>(MissingInclude);
+        << UnknownDeclarationActions(MissingInclude);
     QTest::newRow("unknown_struct") << "" << "" << "test"
-        << static_cast<UnknownDeclarationActions>(ForwardDecls);
+        << UnknownDeclarationActions();
 }
 
 void TestAssistants::testUnknownDeclarationAssistant()
@@ -492,16 +573,27 @@ void TestAssistants::testUnknownDeclarationAssistant()
 
     static const auto cppContents = QStringLiteral("%1\nvoid f_u_n_c_t_i_o_n() {\n}");
     Testbed testbed(headerContents, cppContents.arg(globalText), Testbed::NoAutoInclude);
-    const int line = testbed.document(Testbed::CppDoc)->lines() - 1;
+    const auto document = testbed.document(Testbed::CppDoc);
+    QVERIFY(document);
+    const int line = document->lines() - 1;
     testbed.changeDocument(Testbed::CppDoc, Range(line, 0, line, 0), functionText, true);
 
-    if (actions == NoUnknownDeclaration) {
-        QVERIFY(!staticAssistantsManager()->activeAssistant());
+    DUChainReadLocker lock;
+
+    auto topCtx = DUChain::self()->chainForDocument(document->url());
+    QVERIFY(topCtx);
+
+    const auto problems = topCtx->problems();
+
+    if (actions == NoUnknownDeclarationAction) {
+        QVERIFY(!problems.isEmpty());
         return;
     }
 
-    QVERIFY(staticAssistantsManager()->activeAssistant());
-    const auto assistantActions = staticAssistantsManager()->activeAssistant()->actions();
+    auto firstProblem = problems.first();
+    auto assistant = firstProblem->solutionAssistant();
+    QVERIFY(assistant);
+    const auto assistantActions = assistant->actions();
     QStringList actionDescriptions;
     for (auto action: assistantActions) {
         actionDescriptions << action->description();
@@ -523,3 +615,171 @@ void TestAssistants::testUnknownDeclarationAssistant()
         QCOMPARE(hasMissingInclude, static_cast<bool>(actions & MissingInclude));
     }
 }
+
+void TestAssistants::testMoveIntoSource()
+{
+    QFETCH(QString, origHeader);
+    QFETCH(QString, origImpl);
+    QFETCH(QString, newHeader);
+    QFETCH(QString, newImpl);
+    QFETCH(QualifiedIdentifier, id);
+
+    TestFile header(origHeader, "h");
+    TestFile impl("#include \"" + header.url().byteArray() + "\"\n" + origImpl, "cpp", &header);
+
+    {
+        TopDUContext* headerCtx = nullptr;
+        {
+            DUChainReadLocker lock;
+            headerCtx = DUChain::self()->chainForDocument(header.url());
+        }
+        // Here is a problem: when launching tests one by one, we can reuse the same tmp file for headers.
+        // But because of document chain for header wasn't unloaded properly in previous run we reuse it here
+        // Therefore when using headerCtx->findDeclarations below we find declarations from the previous launch -> tests fail
+        if (headerCtx) {
+            // TODO: Investigate why this chain doesn't get updated when parsing source file
+            DUChainWriteLocker lock;
+            DUChain::self()->removeDocumentChain(headerCtx);
+        }
+    }
+
+    impl.parse(KDevelop::TopDUContext::AllDeclarationsContextsAndUses);
+    QVERIFY(impl.waitForParsed());
+
+    IndexedDeclaration declaration;
+    {
+        DUChainReadLocker lock;
+        auto headerCtx = DUChain::self()->chainForDocument(header.url());
+        QVERIFY(headerCtx);
+        auto decls = headerCtx->findDeclarations(id);
+        Q_ASSERT(!decls.isEmpty());
+        declaration = IndexedDeclaration(decls.first());
+        QVERIFY(declaration.isValid());
+    }
+    CodeRepresentation::setDiskChangesForbidden(false);
+    ClangRefactoring refactoring;
+    QCOMPARE(refactoring.moveIntoSource(declaration), QString());
+    CodeRepresentation::setDiskChangesForbidden(true);
+
+    QCOMPARE(header.fileContents(), newHeader);
+    QVERIFY(impl.fileContents().endsWith(newImpl));
+}
+
+void TestAssistants::testMoveIntoSource_data()
+{
+    QTest::addColumn<QString>("origHeader");
+    QTest::addColumn<QString>("origImpl");
+    QTest::addColumn<QString>("newHeader");
+    QTest::addColumn<QString>("newImpl");
+    QTest::addColumn<QualifiedIdentifier>("id");
+
+    const QualifiedIdentifier fooId("foo");
+
+    QTest::newRow("globalfunction") << QString("int foo()\n{\n    int i = 0;\n    return 0;\n}\n")
+                                    << QString()
+                                    << QString("int foo();\n")
+                                    << QString("\nint foo()\n{\n    int i = 0;\n    return 0;\n}\n")
+                                    << fooId;
+
+    QTest::newRow("staticfunction") << QString("static int foo()\n{\n    int i = 0;\n    return 0;\n}\n")
+                                    << QString()
+                                    << QString("static int foo();\n")
+                                    << QString("\nint foo()\n{\n    int i = 0;\n    return 0;\n}\n")
+                                    << fooId;
+
+    QTest::newRow("funcsameline") << QString("int foo() {\n    int i = 0;\n    return 0;\n}\n")
+                                    << QString()
+                                    << QString("int foo();\n")
+                                    << QString("\nint foo() {\n    int i = 0;\n    return 0;\n}\n")
+                                    << fooId;
+
+    QTest::newRow("func-comment") << QString("int foo()\n/* foobar */ {\n    int i = 0;\n    return 0;\n}\n")
+                                    << QString()
+                                    << QString("int foo()\n/* foobar */;\n")
+                                    << QString("\nint foo() {\n    int i = 0;\n    return 0;\n}\n")
+                                    << fooId;
+
+    QTest::newRow("func-comment2") << QString("int foo()\n/*asdf*/\n{\n    int i = 0;\n    return 0;\n}\n")
+                                    << QString()
+                                    << QString("int foo()\n/*asdf*/;\n")
+                                    << QString("\nint foo()\n{\n    int i = 0;\n    return 0;\n}\n")
+                                    << fooId;
+
+    const QualifiedIdentifier aFooId("a::foo");
+    QTest::newRow("class-method") << QString("class a {\n    int foo(){\n        return 0;\n    }\n};\n")
+                                    << QString()
+                                    << QString("class a {\n    int foo();\n};\n")
+                                    << QString("\nint a::foo() {\n        return 0;\n    }\n")
+                                    << aFooId;
+
+    QTest::newRow("class-method-const") << QString("class a {\n    int foo() const\n    {\n        return 0;\n    }\n};\n")
+                                    << QString()
+                                    << QString("class a {\n    int foo() const;\n};\n")
+                                    << QString("\nint a::foo() const\n    {\n        return 0;\n    }\n")
+                                    << aFooId;
+
+    QTest::newRow("class-method-const-sameline") << QString("class a {\n    int foo() const{\n        return 0;\n    }\n};\n")
+                                    << QString()
+                                    << QString("class a {\n    int foo() const;\n};\n")
+                                    << QString("\nint a::foo() const {\n        return 0;\n    }\n")
+                                    << aFooId;
+    QTest::newRow("elaborated-type") << QString("namespace NS{class C{};} class a {\nint foo(const NS::C c) const{\nreturn 0;\n}\n};\n")
+                                    << QString()
+                                    << QString("namespace NS{class C{};} class a {\nint foo(const NS::C c) const;\n};\n")
+                                    << QString("\nint a::foo(const NS::C c) const {\nreturn 0;\n}\n")
+                                    << aFooId;
+    QTest::newRow("add-into-namespace") << QString("namespace NS{class a {\nint foo() const {\nreturn 0;\n}\n};\n}")
+                                    << QString("namespace NS{\n}")
+                                    << QString("namespace NS{class a {\nint foo() const;\n};\n}")
+                                    << QString("namespace NS{\n\nint a::foo() const {\nreturn 0;\n}\n}")
+                                    << QualifiedIdentifier("NS::a::foo");
+    QTest::newRow("class-template-parameter")
+        << QString(R"(
+            namespace first {
+            template <typename T>
+            class Test{};
+
+            namespace second {
+                template <typename T>
+                class List;
+            }
+
+            class MoveIntoSource
+            {
+            public:
+                void f(const second::List<const volatile Test<first::second::List<int*>>*>& param){}
+            };}
+        )")
+        << QString("")
+        << QString(R"(
+            namespace first {
+            template <typename T>
+            class Test{};
+
+            namespace second {
+                template <typename T>
+                class List;
+            }
+
+            class MoveIntoSource
+            {
+            public:
+                void f(const second::List<const volatile Test<first::second::List<int*>>*>& param);
+            };}
+        )")
+        << QString("namespace first {\nvoid MoveIntoSource::f(const first::second::List< const volatile first::Test< first::second::List< int* > >* >& param) {}}\n\n")
+        << QualifiedIdentifier("first::MoveIntoSource::f");
+
+        QTest::newRow("move-unexposed-type")
+            << QString("namespace std { template<typename _CharT> class basic_string; \ntypedef basic_string<char> string;}\n void move(std::string i){}")
+            << QString("")
+            << QString("namespace std { template<typename _CharT> class basic_string; \ntypedef basic_string<char> string;}\n void move(std::string i);")
+            << QString("void move(std::string i) {}\n")
+            << QualifiedIdentifier("move");
+        QTest::newRow("move-constructor")
+            << QString("class Class{Class(){}\n};")
+            << QString("")
+            << QString("class Class{Class();\n};")
+            << QString("Class::Class() {}\n")
+            << QualifiedIdentifier("Class::Class");
+}
