@@ -32,11 +32,16 @@
 #include <language/duchain/parsingenvironment.h>
 #include <language/duchain/problem.h>
 #include <language/duchain/types/integraltype.h>
+#include <language/duchain/types/structuretype.h>
 #include <language/duchain/types/functiontype.h>
+#include <language/duchain/types/typealiastype.h>
 #include <language/duchain/duchainutils.h>
 #include <language/duchain/classdeclaration.h>
 #include <language/duchain/abstractfunctiondeclaration.h>
 #include <language/duchain/functiondefinition.h>
+#include <language/duchain/classfunctiondeclaration.h>
+#include <language/duchain/forwarddeclaration.h>
+#include <language/duchain/use.h>
 #include <language/backgroundparser/backgroundparser.h>
 #include <interfaces/ilanguagecontroller.h>
 #include <interfaces/idocumentcontroller.h>
@@ -83,6 +88,7 @@ void TestDUChain::initTestCase()
 {
     QLoggingCategory::setFilterRules(QStringLiteral("*.debug=false\ndefault.debug=true\nkdevelop.plugins.clang.debug=true\n"));
     QVERIFY(qputenv("KDEV_DISABLE_PLUGINS", "kdevcppsupport"));
+    QVERIFY(qputenv("KDEV_CLANG_DISPLAY_DIAGS", "1"));
     AutoTestShell::init({QStringLiteral("kdevclangsupport")});
     auto core = TestCore::initialize();
     delete core->projectController();
@@ -186,10 +192,13 @@ void TestDUChain::testElaboratedType()
     auto functionType = function->type<FunctionType>();
     QVERIFY(functionType);
 
+#if CINDEX_VERSION_MINOR < 34
     QEXPECT_FAIL("namespace", "The ElaboratedType is not exposed through the libclang interface, not much we can do here", Abort);
+#endif
     QVERIFY(functionType->returnType()->whichType() != AbstractType::TypeDelayed);
-
+#if CINDEX_VERSION_MINOR < 34
     QEXPECT_FAIL("typedef", "After using clang_getCanonicalType on ElaboratedType all typedef information get's stripped away", Continue);
+#endif
     QCOMPARE(functionType->returnType()->whichType(), type);
 }
 
@@ -238,6 +247,74 @@ void TestDUChain::testInclude()
     QCOMPARE(foo->uses().begin()->first(), RangeInRevision(1, 20, 1, 23));
 }
 
+void TestDUChain::testMissingInclude()
+{
+    auto code = R"(
+#pragma once
+#include "missing1.h"
+
+template<class T>
+class A
+{
+    T a;
+};
+
+#include "missing2.h"
+
+class B : public A<int>
+{
+};
+    )";
+
+    // NOTE: This fails and needs fixing. If the include of "missing2.h"
+    //       above is commented out, then it doesn't fail. Maybe
+    //       clang stops processing when it encounters the second missing
+    //       header, or similar.
+
+    TestFile header(code, "h");
+    TestFile impl("#include \"" + header.url().byteArray() + "\"\n", "cpp", &header);
+    QVERIFY(impl.parseAndWait(TopDUContext::AllDeclarationsContextsAndUses));
+
+    DUChainReadLocker lock;
+
+    auto top = impl.topContext();
+    QVERIFY(top);
+
+    QCOMPARE(top->importedParentContexts().count(), 1);
+
+    TopDUContext* headerCtx = dynamic_cast<TopDUContext*>(top->importedParentContexts().first().context(top));
+    QVERIFY(headerCtx);
+    QCOMPARE(headerCtx->url(), header.url());
+
+#if CINDEX_VERSION_MINOR < 34
+    QEXPECT_FAIL("", "Second missing header isn't reported", Continue);
+#endif
+    QCOMPARE(headerCtx->problems().count(), 2);
+
+    QCOMPARE(headerCtx->localDeclarations().count(), 2);
+
+    auto a = dynamic_cast<ClassDeclaration*>(headerCtx->localDeclarations().first());
+    QVERIFY(a);
+
+    auto b = dynamic_cast<ClassDeclaration*>(headerCtx->localDeclarations().last());
+    QVERIFY(b);
+
+#if CINDEX_VERSION_MINOR < 34
+    QEXPECT_FAIL("", "Base class isn't assigned correctly", Continue);
+#endif
+    QCOMPARE(b->baseClassesSize(), 1u);
+
+#if CINDEX_VERSION_MINOR < 34
+    // at least the one problem we have should have been propagated
+    QCOMPARE(top->problems().count(), 1);
+#else
+    // two errors:
+    // /tmp/testfile_f32415.h:3:10: error: 'missing1.h' file not found
+    // /tmp/testfile_f32415.h:11:10: error: 'missing2.h' file not found
+    QCOMPARE(top->problems().count(), 2);
+#endif
+}
+
 QByteArray createCode(const QByteArray& prefix, const int functions)
 {
     QByteArray code;
@@ -584,15 +661,24 @@ void TestDUChain::testReparseBaseClassesTemplates()
     }
 }
 
-// TODO: Move this test to kdevplatform.
+void TestDUChain::testGetInheriters_data()
+{
+    QTest::addColumn<QString>("code");
+
+    QTest::newRow("inline") << "struct Base { struct Inner {}; }; struct Inherited : Base, Base::Inner {};";
+    QTest::newRow("outline") << "struct Base { struct Inner; }; struct Base::Inner {}; struct Inherited : Base, Base::Inner {};";
+}
+
 void TestDUChain::testGetInheriters()
 {
-    TestFile file("class Base { class Inner {}; }; class Inherited : public Base, Base::Inner {};", "cpp");
+    QFETCH(QString, code);
+    TestFile file(code, "cpp");
     QVERIFY(file.parseAndWait());
 
     DUChainReadLocker lock;
-    DUContext* top = file.topContext().data();
+    auto top = file.topContext();
     QVERIFY(top);
+    QVERIFY(top->problems().isEmpty());
 
     QCOMPARE(top->localDeclarations().count(), 2);
     Declaration* baseDecl = top->localDeclarations().first();
@@ -604,14 +690,17 @@ void TestDUChain::testGetInheriters()
 
     Declaration* innerDecl = baseCtx->localDeclarations().first();
     QCOMPARE(innerDecl->identifier(), Identifier("Inner"));
+    if (auto forward = dynamic_cast<ForwardDeclaration*>(innerDecl)) {
+        innerDecl = forward->resolve(top);
+    }
+    QVERIFY(dynamic_cast<ClassDeclaration*>(innerDecl));
 
-    Declaration* inheritedDecl = top->localDeclarations()[1];
+    Declaration* inheritedDecl = top->localDeclarations().last();
     QVERIFY(inheritedDecl);
     QCOMPARE(inheritedDecl->identifier(), Identifier("Inherited"));
 
     uint maxAllowedSteps = uint(-1);
     auto baseInheriters = DUChainUtils::getInheriters(baseDecl, maxAllowedSteps);
-    QEXPECT_FAIL("", "not yet working properly, tentative fix is up for review for kdevplatform", Abort);
     QCOMPARE(baseInheriters, QList<Declaration*>() << inheritedDecl);
 
     maxAllowedSteps = uint(-1);
@@ -822,7 +911,11 @@ void TestDUChain::testActiveDocumentHasASTAttached()
         url = ctx->url().toUrl();
     }
 
-    // Here the file is already deleted, so clang_parseTranslationUnit2 will fail, but we still get ParseSessionData attached.
+    QVERIFY(!QFileInfo::exists(url.toLocalFile()));
+    QFile file(url.toLocalFile());
+    file.open(QIODevice::WriteOnly);
+    Q_ASSERT(file.isOpen());
+
     auto document = ICore::self()->documentController()->openDocument(url);
     QVERIFY(document);
     ICore::self()->documentController()->activateDocument(document);
@@ -843,6 +936,42 @@ void TestDUChain::testActiveDocumentHasASTAttached()
     DUChain::self()->removeDocumentChain(indexed.data());
 }
 
+void TestDUChain::testActiveDocumentsGetBestPriority()
+{
+    // note: this test would make more sense in kdevplatform, but we don't have a language plugin available there
+    // (required for background parsing)
+    // TODO: Create a fake-language plugin in kdevplatform for testing purposes, use that.
+
+    TestFile file1("int main() {}\n", "cpp");
+    TestFile file2("int main() {}\n", "cpp");
+    TestFile file3("int main() {}\n", "cpp");
+
+    DUChain::self()->storeToDisk();
+
+    auto backgroundParser = ICore::self()->languageController()->backgroundParser();
+    QVERIFY(!backgroundParser->isQueued(file1.url()));
+
+    auto documentController = ICore::self()->documentController();
+
+    // open first document (no activation)
+    auto doc = documentController->openDocument(file1.url().toUrl(), KTextEditor::Range::invalid(), {IDocumentController::DoNotActivate});
+    QVERIFY(doc);
+    QVERIFY(backgroundParser->isQueued(file1.url()));
+    QCOMPARE(backgroundParser->priorityForDocument(file1.url()), (int)BackgroundParser::NormalPriority);
+
+    // open second document, activate
+    doc = documentController->openDocument(file2.url().toUrl());
+    QVERIFY(doc);
+    QVERIFY(backgroundParser->isQueued(file2.url()));
+    QCOMPARE(backgroundParser->priorityForDocument(file2.url()), (int)BackgroundParser::BestPriority);
+
+    // open third document, activate, too
+    doc = documentController->openDocument(file3.url().toUrl());
+    QVERIFY(doc);
+    QVERIFY(backgroundParser->isQueued(file3.url()));
+    QCOMPARE(backgroundParser->priorityForDocument(file3.url()), (int)BackgroundParser::BestPriority);
+}
+
 void TestDUChain::testSystemIncludes()
 {
     ClangParsingEnvironment env;
@@ -1067,6 +1196,146 @@ void TestDUChain::testReparseChangeEnvironment()
     }
 }
 
+void TestDUChain::testMacroDependentHeader()
+{
+    TestFile header("struct MY_CLASS { class Q{Q(); int m;}; int m; };\n", "h");
+    TestFile impl("#define MY_CLASS A\n"
+                  "#include \"" + header.url().byteArray() + "\"\n"
+                  "#undef MY_CLASS\n"
+                  "#define MY_CLASS B\n"
+                  "#include \"" + header.url().byteArray() + "\"\n"
+                  "#undef MY_CLASS\n"
+                  "A a;\n"
+                  "const A::Q aq;\n"
+                  "B b;\n"
+                  "const B::Q bq;\n"
+                  "int am = a.m;\n"
+                  "int aqm = aq.m;\n"
+                  "int bm = b.m;\n"
+                  "int bqm = bq.m;\n"
+                  , "cpp", &header);
+
+    impl.parse(TopDUContext::Features(TopDUContext::AllDeclarationsContextsAndUses|TopDUContext::AST|TopDUContext::ForceUpdate));
+    QVERIFY(impl.waitForParsed(500000));
+
+    DUChainReadLocker lock;
+    TopDUContext* top = impl.topContext().data();
+    QVERIFY(top);
+    QCOMPARE(top->localDeclarations().size(), 10); // 2x macro, then a, aq, b, bq
+    QCOMPARE(top->importedParentContexts().size(), 1);
+    AbstractType::Ptr type = top->localDeclarations()[2]->abstractType();
+    StructureType* sType = dynamic_cast<StructureType*>(type.data());
+    QVERIFY(sType);
+    QCOMPARE(sType->toString(), QString("A"));
+    Declaration* decl = sType->declaration(top);
+    QVERIFY(decl);
+    AbstractType::Ptr type2 = top->localDeclarations()[4]->abstractType();
+    StructureType* sType2 = dynamic_cast<StructureType*>(type2.data());
+    QVERIFY(sType2);
+    QCOMPARE(sType2->toString(), QString("B"));
+    Declaration* decl2 = sType2->declaration(top);
+    QVERIFY(decl2);
+
+    TopDUContext* top2 = dynamic_cast<TopDUContext*>(top->importedParentContexts()[0].context(top));
+    QVERIFY(top2);
+    QCOMPARE(top2->localDeclarations().size(), 2);
+    QCOMPARE(top2->localDeclarations()[0], decl);
+    QCOMPARE(top2->localDeclarations()[1], decl2);
+    qDebug() << "DECL RANGE:" << top2->localDeclarations()[0]->range().castToSimpleRange();
+    qDebug() << "CTX RANGE:" << top2->localDeclarations()[0]->internalContext()->range().castToSimpleRange();
+
+    // validate uses:
+    QCOMPARE(top->usesCount(), 14);
+    QCOMPARE(top->uses()[0].usedDeclaration(top)->qualifiedIdentifier(), QualifiedIdentifier("A"));
+    QCOMPARE(top->uses()[1].usedDeclaration(top)->qualifiedIdentifier(), QualifiedIdentifier("A"));
+    QCOMPARE(top->uses()[2].usedDeclaration(top)->qualifiedIdentifier(), QualifiedIdentifier("A::Q"));
+    QCOMPARE(top->uses()[3].usedDeclaration(top)->qualifiedIdentifier(), QualifiedIdentifier("B"));
+    QCOMPARE(top->uses()[4].usedDeclaration(top)->qualifiedIdentifier(), QualifiedIdentifier("B"));
+    QCOMPARE(top->uses()[5].usedDeclaration(top)->qualifiedIdentifier(), QualifiedIdentifier("B::Q"));
+    QCOMPARE(top->uses()[6].usedDeclaration(top)->qualifiedIdentifier(), QualifiedIdentifier("a"));
+    QCOMPARE(top->uses()[7].usedDeclaration(top)->qualifiedIdentifier(), QualifiedIdentifier("A::m"));
+    QCOMPARE(top->uses()[8].usedDeclaration(top)->qualifiedIdentifier(), QualifiedIdentifier("aq"));
+    QCOMPARE(top->uses()[9].usedDeclaration(top)->qualifiedIdentifier(), QualifiedIdentifier("A::Q::m"));
+    QCOMPARE(top->uses()[10].usedDeclaration(top)->qualifiedIdentifier(), QualifiedIdentifier("b"));
+    QCOMPARE(top->uses()[11].usedDeclaration(top)->qualifiedIdentifier(), QualifiedIdentifier("B::m"));
+    QCOMPARE(top->uses()[12].usedDeclaration(top)->qualifiedIdentifier(), QualifiedIdentifier("bq"));
+    QCOMPARE(top->uses()[13].usedDeclaration(top)->qualifiedIdentifier(), QualifiedIdentifier("B::Q::m"));
+}
+
+void TestDUChain::testHeaderParsingOrder1()
+{
+    TestFile header("typedef const A<int> B;\n", "h");
+    TestFile impl("template<class T> class A{};\n"
+                  "#include \"" + header.url().byteArray() + "\"\n"
+                  "B c;", "cpp", &header);
+
+    impl.parse(TopDUContext::Features(TopDUContext::AllDeclarationsContextsAndUses|TopDUContext::AST|TopDUContext::ForceUpdate));
+    QVERIFY(impl.waitForParsed(500000));
+
+    DUChainReadLocker lock;
+    TopDUContext* top = impl.topContext().data();
+    QVERIFY(top);
+    QCOMPARE(top->localDeclarations().size(), 2);
+    QCOMPARE(top->importedParentContexts().size(), 1);
+    AbstractType::Ptr type = top->localDeclarations()[1]->abstractType();
+    TypeAliasType* aType = dynamic_cast<TypeAliasType*>(type.data());
+    QVERIFY(aType);
+    AbstractType::Ptr targetType = aType->type();
+    QVERIFY(targetType);
+    IdentifiedType *idType = dynamic_cast<IdentifiedType*>(targetType.data());
+    QVERIFY(idType);
+    // this declaration could be resolved, because it was created with an
+    // indirect DeclarationId that is resolved from the perspective of 'top'
+    Declaration* decl = idType->declaration(top);
+    // NOTE: the decl. doesn't know (yet) about the template insantiation <int>
+    QVERIFY(decl);
+    QCOMPARE(decl, top->localDeclarations()[0]);
+    
+    // now ensure that a use was build for 'A' in header1
+    TopDUContext* top2 = dynamic_cast<TopDUContext*>(top->importedParentContexts()[0].context(top));
+    QVERIFY(top2);
+    QEXPECT_FAIL("", "the use could not be created because the corresponding declaration didn't exist yet", Continue);
+    QCOMPARE(top2->usesCount(), 1);
+    // Declaration* decl2 = top2->uses()[0].usedDeclaration(top2);
+    // QVERIFY(decl2);
+    // QCOMPARE(decl, decl2);
+}
+
+void TestDUChain::testHeaderParsingOrder2()
+{
+    TestFile header("template<class T> class A{};\n", "h");
+    TestFile header2("typedef const A<int> B;\n", "h");
+    TestFile impl("#include \"" + header.url().byteArray() + "\"\n"
+                  "#include \"" + header2.url().byteArray() + "\"\n"
+                  "B c;", "cpp", &header);
+
+    impl.parse(TopDUContext::Features(TopDUContext::AllDeclarationsContextsAndUses|TopDUContext::AST|TopDUContext::ForceUpdate));
+    QVERIFY(impl.waitForParsed(500000));
+
+    DUChainReadLocker lock;
+    TopDUContext* top = impl.topContext().data();
+    QVERIFY(top);
+    QCOMPARE(top->localDeclarations().size(), 1);
+    QCOMPARE(top->importedParentContexts().size(), 2);
+    AbstractType::Ptr type = top->localDeclarations()[0]->abstractType();
+    TypeAliasType* aType = dynamic_cast<TypeAliasType*>(type.data());
+    QVERIFY(aType);
+    AbstractType::Ptr targetType = aType->type();
+    QVERIFY(targetType);
+    IdentifiedType *idType = dynamic_cast<IdentifiedType*>(targetType.data());
+    QVERIFY(idType);
+    Declaration* decl = idType->declaration(top);
+    // NOTE: the decl. doesn't know (yet) about the template insantiation <int>
+    QVERIFY(decl);
+    
+    // now ensure that a use was build for 'A' in header2
+    TopDUContext* top2 = dynamic_cast<TopDUContext*>(top->importedParentContexts()[1].context(top));
+    QVERIFY(top2);
+    QCOMPARE(top2->usesCount(), 1);
+    Declaration* decl2 = top2->uses()[0].usedDeclaration(top2);
+    QCOMPARE(decl, decl2);
+}
+
 void TestDUChain::testMacrosRanges()
 {
     TestFile file("#define FUNC_MACROS(x) struct str##x{};\nFUNC_MACROS(x);", "cpp");
@@ -1087,6 +1356,28 @@ void TestDUChain::testMacrosRanges()
     QCOMPARE(macroDefinition->uses().begin()->first(), RangeInRevision(1,0,1,11));
 }
 
+void TestDUChain::testMacroUses()
+{
+    TestFile file("#define USER(x) x\n#define USED\nUSER(USED)", "cpp");
+    file.parse(TopDUContext::AllDeclarationsContextsAndUses);
+    QVERIFY(file.waitForParsed(5000));
+
+    DUChainReadLocker lock;
+    QVERIFY(file.topContext());
+    QCOMPARE(file.topContext()->localDeclarations().size(), 2);
+    auto macroDefinition1 = file.topContext()->localDeclarations()[0];
+    auto macroDefinition2 = file.topContext()->localDeclarations()[1];
+
+    QCOMPARE(macroDefinition1->uses().size(), 1);
+    QCOMPARE(macroDefinition1->uses().begin()->first(), RangeInRevision(2,0,2,4));
+    QEXPECT_FAIL("", "This appears to be a clang bug, the AST doesn't contain the macro use", Continue);
+    QCOMPARE(macroDefinition2->uses().size(), 1);
+    if (macroDefinition2->uses().size())
+    {
+        QCOMPARE(macroDefinition2->uses().begin()->first(), RangeInRevision(2,5,2,9));
+    }
+}
+
 void TestDUChain::testMultiLineMacroRanges()
 {
     TestFile file("#define FUNC_MACROS(x) struct str##x{};\nFUNC_MACROS(x\n);", "cpp");
@@ -1327,6 +1618,25 @@ void TestDUChain::testUsesCreatedForDeclarations()
     QCOMPARE(functionDeclaration->uses().count(), 1);
 }
 
+void TestDUChain::testReparseIncludeGuard()
+{
+    TestFile header("#ifndef GUARD\n#define GUARD\nint something;\n#endif\n", "h");
+    TestFile impl("#include \"" + header.url().byteArray() + "\"\n", "cpp", &header);
+
+    impl.parseAndWait(TopDUContext::Features(TopDUContext::AllDeclarationsContextsAndUses | TopDUContext::AST  ));
+    {
+        DUChainReadLocker lock;
+        QCOMPARE(static_cast<TopDUContext*>(impl.topContext()->
+            importedParentContexts().first().context(impl.topContext()))->problems().size(), 0);
+    }
+    impl.parseAndWait(TopDUContext::Features(TopDUContext::AllDeclarationsContextsAndUses | TopDUContext::ForceUpdateRecursive));
+    {
+        DUChainReadLocker lock;
+        QCOMPARE(static_cast<TopDUContext*>(impl.topContext()->
+            importedParentContexts().first().context(impl.topContext()))->problems().size(), 0);
+    }
+}
+
 void TestDUChain::testExternC()
 {
     auto code = R"(extern "C" { void foo(); })";
@@ -1391,7 +1701,7 @@ void TestDUChain::testReparseUnchanged()
         QVERIFY(headerCtx->problems().isEmpty());
         auto implCtx = DUChain::self()->chainForDocument(impl.url());
         QVERIFY(implCtx);
-        if (reparsed && CINDEX_VERSION_MINOR > 29) {
+        if (reparsed && CINDEX_VERSION_MINOR > 29 && CINDEX_VERSION_MINOR < 33) {
             QEXPECT_FAIL("template-default-parameters", "the precompiled preamble messes the default template parameters up in clang 3.7", Continue);
         }
         QVERIFY(implCtx->problems().isEmpty());
@@ -1421,6 +1731,36 @@ void TestDUChain::testTypeAliasTemplate()
     QCOMPARE(templateAlias->abstractType()->toString(), QStringLiteral("TypeAliasTemplate"));
 }
 
+void TestDUChain::testDeclarationsInsideMacroExpansion()
+{
+    TestFile header("#define DECLARE(a) typedef struct a##__ {int var;} *a\nDECLARE(D);\n", "h");
+    TestFile file("#include \"" + header.url().byteArray() + "\"\nint main(){\nD d; d->var;}\n", "cpp");
+
+    file.parse(TopDUContext::Features(TopDUContext::AllDeclarationsContextsAndUses|TopDUContext::AST));
+    QVERIFY(file.waitForParsed(5000));
+
+    {
+        DUChainReadLocker lock;
+        QVERIFY(file.topContext());
+    }
+
+    file.parse(TopDUContext::Features(TopDUContext::AllDeclarationsContextsAndUses|TopDUContext::AST|TopDUContext::ForceUpdate));
+    QVERIFY(file.waitForParsed(5000));
+
+    DUChainReadLocker lock;
+    QVERIFY(file.topContext());
+    QCOMPARE(file.topContext()->localDeclarations().size(), 1);
+
+    auto context = file.topContext()->childContexts().first()->childContexts().first();
+    QVERIFY(context);
+    QCOMPARE(context->localDeclarations().size(), 1);
+    QCOMPARE(context->usesCount(), 3);
+
+    QCOMPARE(context->uses()[0].m_range, RangeInRevision({2, 0}, {2, 1}));
+    QCOMPARE(context->uses()[1].m_range, RangeInRevision({2, 5}, {2, 6}));
+    QCOMPARE(context->uses()[2].m_range, RangeInRevision({2, 8}, {2, 11}));
+}
+
 static bool containsErrors(const QList<Problem::Ptr>& problems)
 {
     auto it = std::find_if(problems.begin(), problems.end(), [] (const Problem::Ptr& problem) {
@@ -1476,11 +1816,12 @@ void TestDUChain::testGccCompatibility()
     m_projectController->addProject(project);
 
     {
+        // TODO: Also test in C mode. Currently it doesn't work (some intrinsics missing?)
         TestFile file(R"(
             #include <x86intrin.h>
 
             int main() { return 0; }
-        )", "c", project, dir.path());
+        )", "cpp", project, dir.path());
 
         file.parse();
         QVERIFY(file.waitForParsed(5000));
@@ -1492,3 +1833,65 @@ void TestDUChain::testGccCompatibility()
 
     m_projectController->closeAllProjects();
 }
+
+void TestDUChain::testQtIntegration()
+{
+    QTemporaryDir includeDir;
+    {
+        QDir dir(includeDir.path());
+        dir.mkdir("QtCore");
+        // create the file but don't put anything in it
+        QFile header(includeDir.path() + "/QtCore/qobjectdefs.h");
+        QVERIFY(header.open(QIODevice::WriteOnly | QIODevice::Text));
+    }
+    QTemporaryDir dir;
+    auto project = new TestProject(Path(dir.path()), this);
+    m_provider->defines.clear();
+    m_provider->includes = {Path(includeDir.path() + "/QtCore")};
+
+    m_projectController->addProject(project);
+
+    {
+        TestFile file(R"(
+            #define slots
+            #define signals
+            #define Q_SLOTS
+            #define Q_SIGNALS
+            #include <QtCore/qobjectdefs.h>
+
+            struct MyObject {
+            public:
+              void other1();
+            public slots:
+              void slot1();
+            signals:
+              void signal1();
+            private Q_SLOTS:
+              void slot2();
+            Q_SIGNALS:
+              void signal2();
+            public:
+              void other2();
+            };
+        )", "cpp", project, dir.path());
+
+        file.parse();
+        QVERIFY(file.waitForParsed(5000));
+
+        DUChainReadLocker lock;
+        auto top = file.topContext();
+        QVERIFY(top);
+        QVERIFY(top->problems().isEmpty());
+        const auto methods = top->childContexts().last()->localDeclarations();
+        QCOMPARE(methods.size(), 6);
+        foreach(auto method, methods) {
+            auto classFunction = dynamic_cast<ClassFunctionDeclaration*>(method);
+            QVERIFY(classFunction);
+            auto id = classFunction->identifier().toString();
+            QCOMPARE(classFunction->isSignal(), id.startsWith(QLatin1String("signal")));
+            QCOMPARE(classFunction->isSlot(), id.startsWith(QLatin1String("slot")));
+        }
+    }
+
+    m_projectController->closeAllProjects();
+}
