@@ -48,6 +48,7 @@ private slots:
     void testElaboratedType_data();
 
     void testInclude();
+    void testMissingInclude();
     void testIncludeLocking();
     void testReparse();
     void testReparseError();
@@ -60,6 +61,7 @@ private slots:
     void testReparseBaseClasses();
     void testReparseBaseClassesTemplates();
     void testGetInheriters();
+    void testGetInheriters_data();
     void testGlobalFunctionDeclaration();
     void testFunctionDefinitionVsDeclaration();
     void testEnsureNoDoubleVisit();
@@ -70,6 +72,10 @@ private slots:
     void testReparseInclude();
     void testReparseChangeEnvironment();
     void testMacrosRanges();
+    void testMacroUses();
+    void testHeaderParsingOrder1();
+    void testHeaderParsingOrder2();
+    void testMacroDependentHeader();
     void testNestedImports();
     void testEnvironmentWithDifferentOrderOfElements();
     void testReparseMacro();
@@ -78,14 +84,18 @@ private slots:
     void testGotoStatement();
     void testRangesOfOperatorsInsideMacro();
     void testActiveDocumentHasASTAttached();
+    void testActiveDocumentsGetBestPriority();
     void testUsesCreatedForDeclarations();
+    void testReparseIncludeGuard();
     void testExternC();
     void testReparseUnchanged_data();
     void testReparseUnchanged();
     void testTypeAliasTemplate();
+    void testDeclarationsInsideMacroExpansion();
 
     void benchDUChainBuilder();
     void testGccCompatibility();
+    void testQtIntegration();
 
 private:
     QScopedPointer<TestEnvironmentProvider> m_provider;
