@@ -29,11 +29,11 @@
 #include <language/codecompletion/codecompletioncontext.h>
 
 namespace KDevelop {
-  class DUContext;
-  class Declaration;
-  class CodeCompletionContext;
-  class CompletionTreeElement;
-  class ParseJob;
+class DUContext;
+class Declaration;
+class CodeCompletionContext;
+class CompletionTreeElement;
+class ReferencedTopDUContext;
 }
 
 namespace Cpp {
@@ -66,7 +66,7 @@ class CodeCompletionModel : public KDevelop::CodeCompletionModel
                                    const QExplicitlySharedDataPointer<KDevelop::CodeCompletionContext>& completionContext) override;
 
   private slots:
-    void parseJobFinished(KDevelop::ParseJob* job);
+    void updateReady(const KDevelop::IndexedString& document, const KDevelop::ReferencedTopDUContext& context);
 
   private:
     static CodeCompletionModel* s_self;
