@@ -20,10 +20,10 @@
 #include "dumptree.h"
 #include "lexer.h"
 #include "parsesession.h"
+#include "debug.h"
 
 #include <QtCore/QString>
 
-#include <kdebug.h>
 #include "rpp/chartools.h"
 
 char const * const names[] = {
@@ -132,11 +132,6 @@ void DumpTree::dump( AST * node, class TokenStream * tokenStream, bool forceOutp
   m_tokenStream = 0;
 }
 
-inline QDebug debug(bool forceOutput)
-{
-  return forceOutput ? qDebug() : kDebugStream(QtDebugMsg, 9007);
-}
-
 void DumpTree::visit(AST *node)
 {
   if (!node) {
@@ -151,14 +146,14 @@ void DumpTree::visit(AST *node)
     }
   }
 
-  debug(m_forceOutput) << QString(indent * 2, ' ').toLatin1().constData() << names[node->kind]
+  qCDebug(CPPPARSER) << QString(indent * 2, ' ').toLatin1().constData() << names[node->kind]
                <<  "[" << node->start_token << "," << node->end_token << "]" << nodeText;
 
   ++indent;
   DefaultVisitor::visit(node);
   --indent;
 
-  debug(m_forceOutput) << QString(indent * 2, ' ').toLatin1().constData() << names[node->kind];
+  qCDebug(CPPPARSER) << QString(indent * 2, ' ').toLatin1().constData() << names[node->kind];
 }
 
 DumpTree::~ DumpTree( )
