@@ -18,15 +18,15 @@
  * 02110-1301, USA.
  */
 
-#ifndef MAKEBUILDERPREFERENCES_H
-#define MAKEBUILDERPREFERENCES_H
+#ifndef CMAKEBUILDERPREFERENCES_H
+#define CMAKEBUILDERPREFERENCES_H
 
 #include <interfaces/configpage.h>
 
 class QWidget;
 class QStringList;
 class CMakeBuilderSettings;
-namespace Ui { class CMakeBuilderConfig; }
+namespace Ui { class CMakeBuilderPreferences; }
 
 class CMakeBuilderPreferences : public KDevelop::ConfigPage
 {
@@ -34,21 +34,14 @@ class CMakeBuilderPreferences : public KDevelop::ConfigPage
 
 public:
     explicit CMakeBuilderPreferences(KDevelop::IPlugin* plugin, QWidget* parent = nullptr);
-    ~CMakeBuilderPreferences();
+    ~CMakeBuilderPreferences() override;
 
-    virtual QString name() const override;
-    virtual QString fullName() const override;
-    virtual QIcon icon() const override;
-
-    virtual void apply() override;
-    virtual void reset() override;
-    virtual void defaults() override;
-
-public slots:
-    void generatorChanged(const QString& generator);
+    QString name() const override;
+    QString fullName() const override;
+    QIcon icon() const override;
 
 private:
-    Ui::CMakeBuilderConfig* m_prefsUi;
+    Ui::CMakeBuilderPreferences* m_prefsUi;
 };
 
 #endif
