@@ -35,6 +35,7 @@
 #include "testing/ctestutils.h"
 
 #include <QDir>
+#include <QReadWriteLock>
 #include <QThread>
 #include <QFileSystemWatcher>
 #include <QTimer>
@@ -89,6 +90,7 @@ CMakeManager::CMakeManager( QObject* parent, const QVariantList& )
     KDEV_USE_EXTENSION_INTERFACE( ICMakeManager)
 
     if (hasError()) {
+        m_highlight = nullptr;
         return;
     }
 
@@ -116,7 +118,11 @@ QString CMakeManager::errorDescription() const
 }
 
 CMakeManager::~CMakeManager()
-{}
+{
+    parseLock()->lockForWrite();
+    // By locking the parse-mutexes, we make sure that parse jobs get a chance to finish in a good state
+    parseLock()->unlock();
+}
 
 bool CMakeManager::hasIncludesOrDefines(ProjectBaseItem* item) const
 {
@@ -268,10 +274,19 @@ static void populateTargets(ProjectFolderItem* folder, const QHash<KDevelop::Pat
         }
     }
 
+    static QSet<QString> standardTargets = {
+        QStringLiteral("edit_cache"), QStringLiteral("rebuild_cache"),
+        QStringLiteral("list_install_components"),
+        QStringLiteral("test"), //not really standard, but applicable for make and ninja
+        QStringLiteral("install")
+
+    };
+
     foreach (const QString& name, dirTargets) {
-        if (!name.endsWith("_automoc")
-            && name != QLatin1String("edit_cache")
-            && name != QLatin1String("rebuild_cache"))
+        if (!name.endsWith(QLatin1String("_automoc"))
+            && !standardTargets.contains(name)
+            && !name.startsWith(QLatin1String("install/"))
+        )
             new CMakeTargetItem(folder, name);
     }
 
